﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using System.Linq;
using System.Web.Http;
using System.Text.RegularExpressions;
using Utils;

namespace TpmAPI.Controllers
{
    [RoutePrefix("api/access")]
    public class AccessController : BaseController
    {
        [HttpPut]
        [Route("register")]
        public HttpResponseMessage RegisterUser(User User)
        {
            var Response = new HttpResponseMessage();
            try
            {
                #region State, City and Hand Treatment

                var City = AddressDAO.FindCityByName(User.Person.Address.City.NameCity);
                User.Person.Address.IdCity = City.IdCity;

                User.Person.Address.City.State = null;
                User.Person.Address.City = null;
                User.Person.IdHand = User.Person.Hand.IdHand;
                User.Person.Hand = null;
                User.Person.TotalStats = new TotalStats();

                #endregion

                #region Role

                var UserRole = new UserRole
                {
                    User = User,
                    IdRole = 3,
                    IdUser = User.IdUser
                };

                User.UserRoles = new List<UserRole>
                {
                    UserRole
                };

                #endregion

                UserDAO.Insert(User);

                return SuccessTreatment(User, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Message Here");
            }
        }

        [HttpGet]
        [Route("checkEmail")]
        public HttpResponseMessage CheckEmail(string Email)
        {
            try
            {
                var User = UserDAO.FindByEmail(Email);
                if (User != null)
                    return CustomResponse("Usuário já cadastrado", Models.Type.error, Request);
                else
                    return CustomResponse("E-mail disponível", Models.Type.success, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Erro ao verificar E-mail");
            }
        }

        [HttpGet]
        [Route("login")]
        public HttpResponseMessage Login(string Email, string Password)
        {
            try
            {
                var User = UserDAO.Autenticate(Email, Password);
                if (User != null)
                {
                    if (User.UserRoles.Where(x => x.IdRole != 1 && x.Status).ToList().Count > 0)
                    {
                        User.UserRoles = null;
                        return SuccessTreatment(User, Request);
                    }
                    foreach (var item in User.UserRoles)
                    {
                        if (item.Status)
                        {
                            User.Roles = new List<Role>
                            {
                                RoleDAO.FindByKey(item.IdRole)
                            };
                        }
                    }
                }
                return CustomResponse("Usuário e/ou senha incorreto(s)", Models.Type.error, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Erro ao fazer login");
            }
        }

        [HttpGet]
        [Route("loginFacebook")]
        public HttpResponseMessage LoginFacebook(string Email)
        {
            try
            {
                var User = UserDAO.FindByEmail(Email);
                if (User != null)
                {
                    if (User.UserRoles.Where(x => x.IdRole != 1 && x.Status).ToList().Count > 0)
                    {
                        User.UserRoles = null;
                        return SuccessTreatment(User, Request);
                    }
                    foreach (var item in User.UserRoles)
                    {
                        if (item.Status)
                        {
                            User.Roles = new List<Role>
                            {
                                RoleDAO.FindByKey(item.IdRole)
                            };
                        }
                    }
                }
                return CustomResponse("E-mail disponível", Models.Type.success, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Erro ao verificar E-mail");
            }
        }

        [HttpGet]
        [Route("forgotPassword")]
        public HttpResponseMessage ForgotPassword(string Email)
        {
            try
            {
                var User = UserDAO.FindByEmail(Email);
                if (User != null)
                {
                    if (User.FacebookId != null)
                        return CustomResponse("Utilize o login com Facebook", Models.Type.info, Request);
                    else
                    {
                        User.Token = Token.GenerateSafeToken();
                        UserDAO.Update(User);
                        SmtpServer.SendEmailResetPassword(User.Email, PersonDAO.FindByKey(User.IdPerson).FullName, User.Token);
                        return CustomResponse("Foi enviado um link de redefinição de senha ao seu email!", Models.Type.success, Request);
                    }
                }
                return CustomResponse("Email não encontrado", Models.Type.error, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Erro ao enviar token de redefinição de senha!");
            }
        }

    }
}
