﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Web;
using System.Web.Http;
using TpmAPI.Models;

namespace TpmAPI.Controllers
{
    public class BaseController : ApiController
    {
        /// <summary>
        /// Success Response for request
        /// </summary>
        /// <param name="Value">Object</param>
        /// <param name="Request">Current request</param>
        /// <returns>Response to request</returns>
        protected static HttpResponseMessage SuccessTreatment(Object Value, HttpRequestMessage Request)
        {
            var Response = new HttpResponseMessage();
            Response = Request.CreateResponse(HttpStatusCode.OK);
            Response.Content = new StringContent(
                JsonConvert.SerializeObject(Value, new JsonSerializerSettings
                { ReferenceLoopHandling = ReferenceLoopHandling.Ignore })
            , Encoding.UTF8, "application/json");
            return Response;
        }

        /// <summary>
        /// Error Response fro Request
        /// </summary>
        /// <param name="e">Exception</param>
        /// <param name="Request">Current request</param>
        /// <param name="Message">Message</param>
        /// <returns>Response to request</returns>
        protected static HttpResponseMessage ExceptionTreatment(Exception e, HttpRequestMessage Request, string Message)
        {
            var Response = new HttpResponseMessage();
            var Inner = e.InnerException != null ? e.InnerException.Message : "";
            Response = Request.CreateResponse(HttpStatusCode.Gone);
            Response.Content = new StringContent(
                JsonConvert.SerializeObject(
                    new FeedbackModel { Type = Models.Type.error, Message = Message + " - " + e.Message + "  " + Inner })
                    , Encoding.UTF8, "application/json");
            return Response;
        }

        /// <summary>
        /// Custom Message Response
        /// </summary>
        /// <param name="Message">Your message</param>
        /// <param name="Request">Current message</param>
        /// <param name="Type">Type enum</param>
        /// <returns>Response to request</returns>
        protected static HttpResponseMessage CustomResponse(string Message, Models.Type Type, HttpRequestMessage Request)
        {
            try
            {
                var Response = new HttpResponseMessage();
                Response = Request.CreateResponse(HttpStatusCode.OK);
                Response.Content = new StringContent(
                     JsonConvert.SerializeObject(
                    new FeedbackModel { Type = Type, Message = Message })
                    , Encoding.UTF8, "application/json");
                return Response;
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Erro ao solicitar requisição");
            }
        }
    }
}