﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using System.Linq;
using System.Web.Http;
using System.Text.RegularExpressions;
using Utils;
using TpmDAO.Entity.Misc.APIReturn;
using Newtonsoft.Json;

namespace TpmAPI.Controllers
{
    [RoutePrefix("api/tournament")]
    public class TournamentController : BaseController
    {
        [HttpPut]
        [Route("nextTournaments")]
        public HttpResponseMessage GetNextTournaments(List<Person> Persons)
        {
            var Response = new HttpResponseMessage();
            try
            {
                var Category = SetCategoryPlayer(Persons[0].BirthDate.Value, Persons[0].Class, Persons[0].Gender);

                var Tournaments = TournamentDAO.FindActiveTournamentsForUser(Category.IdCategory, Persons[0].IdPerson).OrderBy(x => x.StartDate).ToList();

                foreach(Tournament t in Tournaments)
                {
                    t.Prize = CategoriesInTournamentDAO.GetPrizeByCategory(Category.IdCategory, t.IdTournament);
                    t.IsEnrolled = CheckEnrolled(t.IdTournament, Persons[0].IdPerson);
                }

                return SuccessTreatment(Tournaments, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Message Here");
            }
        }

        [HttpPut]
        [Route("nextTournamentsByCity")]
        public HttpResponseMessage GetNextTournamentsByCity(List<Person> Persons)
        {
            var Response = new HttpResponseMessage();
            try
            {
                var Category = SetCategoryPlayer(Persons[0].BirthDate.Value, Persons[0].Class, Persons[0].Gender);

                var Tournaments = TournamentDAO.FindActiveTournamentsForUserByCity(Category.IdCategory, Persons[0].IdPerson, Persons[0].Address.City.NameCity).OrderBy(x => x.StartDate).ToList();

                foreach (Tournament t in Tournaments)
                {
                    t.Prize = CategoriesInTournamentDAO.GetPrizeByCategory(Category.IdCategory, t.IdTournament);
                    t.IsEnrolled = CheckEnrolled(t.IdTournament, Persons[0].IdPerson);
                }

                return SuccessTreatment(Tournaments, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Message Here");
            }
        }

        [HttpPut]
        [Route("enroll")]
        public HttpResponseMessage EnrollPlayer(EnrolledPlayers EnrolledPlayers)
        {
            var Response = new HttpResponseMessage();
            try
            {
                var Category = SetCategoryPlayer(EnrolledPlayers.Person.BirthDate.Value, EnrolledPlayers.Person.Class, EnrolledPlayers.Person.Gender);

                EnrolledPlayers.Category = SetCategoryPlayer(EnrolledPlayers.Person.BirthDate.Value, EnrolledPlayers.Person.Class, EnrolledPlayers.Person.Gender);

                EnrolledPlayers.IdCategory = EnrolledPlayers.Category.IdCategory;
                EnrolledPlayers.IdPerson = EnrolledPlayers.Person.IdPerson;
                EnrolledPlayers.IdTournament = EnrolledPlayers.Tournament.IdTournament;
                EnrolledPlayers.Category = null;
                EnrolledPlayers.Tournament = null;
                EnrolledPlayers.Person = null;

                EnrolledPlayersDAO.Insert(EnrolledPlayers);

                return SuccessTreatment(EnrolledPlayers, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Message Here");
            }
        }

        [HttpPut]
        [Route("cancelEnroll")]
        public HttpResponseMessage CancelEnrollPlayer(EnrolledPlayers EnrolledPlayers)
        {
            var Response = new HttpResponseMessage();
            try
            {
                EnrolledPlayers.IdPerson = EnrolledPlayers.Person.IdPerson;
                EnrolledPlayers.IdTournament = EnrolledPlayers.Tournament.IdTournament;
                EnrolledPlayers.Category = null;
                EnrolledPlayers.Tournament = null;
                EnrolledPlayers.Person = null;
                EnrolledPlayersDAO.Delete(EnrolledPlayers);

                return SuccessTreatment(EnrolledPlayers, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Message Here");
            }
        }

        [HttpPut]
        [Route("enrolledTournaments")]
        public HttpResponseMessage EnrolledTournaments(List<Person> Persons)
        {
            var Response = new HttpResponseMessage();
            try
            {

                var Tournaments = TournamentDAO.FindActiveTournamentsForUserStarted(Persons[0].IdPerson).OrderBy(x => x.Tournament.StartDate).ToList();

                var TournamentsApp = FillTournamentApp(ref Tournaments, Persons.First().IdPerson);


                return SuccessTreatment(TournamentsApp, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Message Here");
            }
        }

        [HttpPut]
        [Route("umpireTournaments")]
        public HttpResponseMessage UmpireTournaments(List<Person> Persons)
        {
            var Response = new HttpResponseMessage();
            try
            {
                var Tournaments = TournamentDAO.FindByIdUmpire(Persons[0].IdPerson);

                return SuccessTreatment(Tournaments, Request);
            }
            catch (Exception e)
            {
                return ExceptionTreatment(e, Request, "Message Here");
            }
        }

        public Category SetCategoryPlayer(DateTime Birthdate, string Class, string Gender)
        {
            try
            {
                var Age = DateTime.Today.Year - Birthdate.Year;

                if (Birthdate > DateTime.Today.AddYears(-Age))
                    Age--;

                return CategoryDAO.GetPlayerCategory(Age, Class, Gender);
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        public bool CheckEnrolled(int IdTournament, int IdPerson)
        {
            try
            {
                var PlayerEnrolled = EnrolledPlayersDAO.FindPlayerEnrolled(IdPerson, IdTournament);

                if (PlayerEnrolled == null)
                    return false;
                return true;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private List<TournamentForApp> FillTournamentApp(ref List<CategoriesInTournament> Tournaments, int IdPerson)
        {
            try
            {
                var TournamentApp = Tournaments.Select(a => new TournamentForApp
                {
                    IdTournament = a.IdTournament,
                    IdCategory = a.IdCategory,
                    Draw = a.IdDraw ?? 0,
                    Category = CategoryDAO.GetDescriptWithOutGender(a.IdCategory),
                    Prize = a.Prize,
                    TotalPlayers = a.Tournament.EnrolledPlayers.Count,
                    TournamentInfo = new TournamentDetails
                    {
                        NameTournament = a.Tournament.NameTournament,
                        Facility = a.Tournament.Facility.NameFacility,
                        StartDate = a.Tournament.StartDate,
                        TournamentPicture = a.Tournament.TournamentPicture,
                        Matches = FillMatches(a.IdTournament, a.IdCategory, IdPerson)
                    },
                }).ToList();

                foreach (var item in TournamentApp)
                {
                    var Draw = DrawDAO.FindByKeyComplete(item.Draw);
                    var Final = Draw.Matches.Where(x => x.IdRound == 1).FirstOrDefault();
                    item.IsOver = Final.MatchStats != null && Final.MatchStats.Count > 0 && Final.MatchStats.Where(x => !string.IsNullOrEmpty(x.Result) && x.Result.Equals("W")).FirstOrDefault() != null ? true : false;
                    var Matches = DrawDAO.FindByTournamentAndCategory(item.IdTournament, item.IdCategory).Matches.Where(q => q.MatchStats.Select(i => i.IdPerson).ToList().Contains(IdPerson)).ToList();

                    var Last = new TpmDAO.Entity.Match();
                    if (item.IsOver)
                    {
                        item.HasAnotherMatch = false;
                        Last = Matches.Where(a => a.MatchStats.Where(x => !string.IsNullOrEmpty(x.Result)).FirstOrDefault() != null).LastOrDefault();
                    }
                    else
                    {
                        if (Matches.Where(a => a.MatchStats.Where(x => !string.IsNullOrEmpty(x.Result) && x.Result.Equals("L")).FirstOrDefault() != null).FirstOrDefault() != null)
                        {
                            Last = Matches.Where(a => a.MatchStats.Where(x => !string.IsNullOrEmpty(x.Result) && x.Result.Equals("L")).FirstOrDefault() != null).FirstOrDefault();
                            item.HasAnotherMatch = false;
                        }
                        else
                        {
                            if (Matches.Count() != 1)
                                Last = Matches.Where(a => a.MatchStats.Where(x => !string.IsNullOrEmpty(x.Result)).FirstOrDefault() != null).FirstOrDefault().NextMatch;
                            else
                                Last = Matches[0];
                            item.HasAnotherMatch = true;
                        }
                    }


                    if (Last != null)
                    {
                        if (Last.IdRound == 1)
                        {
                            if (Last.MatchStats.Where(x => !string.IsNullOrEmpty(x.Result)).FirstOrDefault() != null)
                                item.Result = Last.MatchStats.Where(x => x.Result.Equals("W")).FirstOrDefault() != null ? "Campeão" : "Vice-Campeão";
                            else
                                item.Result = Last.Round.NameRound;
                        }
                        else
                        {
                            item.Result = Last.Round.NameRound;
                        }
                    }
                }

                return TournamentApp;

            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private List<TournamentMatches> FillMatches(int IdTournament, int IdCategory, int IdPerson)
        {
            try
            {
                var Matches = DrawDAO.FindByTournamentAndCategory(IdTournament, IdCategory).Matches.Where(q => q.MatchStats.Select(i => i.IdPerson).ToList().Contains(IdPerson) && (q.MatchStats.Count() == 2 || (q.MatchStats.Count() == 1 && q.MatchStats.Select(r => r.Result) == null))).ToList();
                var oMatches = Matches.Select(q => new TournamentMatches
                {
                    Round = q.Round.NameRound,
                    IdRound = q.IdRound.Value,
                    Surface = q.Surface != null ? q.Surface.NameSurface : "",
                    Stats = q.MatchStats.OrderBy(x => x.IdPerson != IdPerson).ToList(),
                    IsOver = q.MatchStats.Select(x => x.Result).ToList().Contains("W"),
                    Won = q.MatchStats.Where(i => i.IdPerson == IdPerson).Select(x => x.Result).Contains("W"),
                    DateMatch = q.DateMatch ?? null
                }).ToList();

                oMatches.ForEach(x => x.Stats.ForEach(a => a.Match = null));
                oMatches.ForEach(x => x.Stats.ForEach(a => a.Person.UserStats = null));

                return oMatches;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }
    }
}