﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace TpmDAO.Entity
{
    [Table("Person")]
    public class Person
    {
        [DatabaseGenerated(DatabaseGeneratedOption.Identity)]
        [Key, Column("idPerson")]
        public int IdPerson { get; set; }

        [Column("firstName")]
        [MaxLength(50)]
        public string FirstName { get; set; }

        [Column("lastName")]
        [MaxLength(50)]
        public string LastName { get; set; }

        [Column("phone")]
        [MaxLength(11)]
        public string Phone { get; set; }

        [Column("birthDate")]
        public DateTime? BirthDate { get; set; }

        [Column("gender")]
        [MaxLength(1)]
        public string Gender { get; set; }

        [Column("class")]
        [MaxLength(1)]
        public string Class { get; set; }

        [Column("idAddress"), ForeignKey("Address")]
        public int? IdAddress { get; set; }

        [Column("idHand"), ForeignKey("Hand")]
        public int? IdHand { get; set; }

        [NotMapped]
        public string FullName
        {
            get { return FirstName + ' ' + LastName; }
        }

        public virtual Address Address { get; set; }
      
        public virtual Hand Hand { get; set; }

        public virtual TotalStats TotalStats { get; set; }

        [ScriptIgnore]
        public virtual ICollection<Stats> UserStats { get; set; }

        [ScriptIgnore]
        public virtual ICollection<Umpire> Umpires { get; set; }

        [ScriptIgnore]
        public virtual ICollection<Tournament> Tournaments { get; set; }

        [ScriptIgnore]
        public virtual ICollection<UserAchievement> UserAchievements { get; set; }

        [ScriptIgnore]
        public virtual ICollection<EnrolledPlayers> EnrolledPlayers { get; set; }
    }
}
