﻿using System;
using System.Collections.Generic;
using System.Linq;
using TpmDAO.Entity;
using TpmDAO.Generic;
using Utils;

namespace TpmDAO.Persistence
{
    public class CategoriesInTournamentDAO : GenericDAO<CategoriesInTournament, int>
    {
        public static List<CategoriesInTournament> FindAllById(int IdTournament)
        {
            try
            {
                using (Connection Con = new Connection())
                {

                    return Con.DbSetCategory.Join(Con.DbSetCategoryAge, Cat => Cat.IdCategoryAge, Age => Age.IdCategoryAge, (Cat, Age) => new { Cat, Age })
                    .Join(Con.DbSetCategoryClass, Cat => Cat.Cat.IdCategoryClass, Class => Class.IdCategoryClass, (Cat, Class) => new { Cat, Class })
                    .Join(Con.DbSetCategoryGender, Cat => Cat.Cat.Cat.IdCategoryGender, Gen => Gen.IdCategoryGender, (Cat, Gen) => new { Cat, Gen })
                    .Join(Con.DbSetCategoriesInTournament, Cats => Cats.Cat.Cat.Cat.IdCategory, Cats => Cats.IdCategory, (Cat, Cats) => new { Cat, Cats }).Where(x => x.Cats.IdTournament == IdTournament)
                    .ToList().Select(x => new CategoriesInTournament
                    {
                        IdCategory = x.Cats.IdCategory,
                        IdTournament = x.Cats.IdTournament,
                        Prize = x.Cats.Prize,
                        Description = x.Cat.Gen.Description + ", " + x.Cat.Cat.Cat.Age.Description + ", " + x.Cat.Cat.Class.Class,
                        IdDraw = x.Cats.IdDraw,
                        Over = x.Cats.Over
                    }).ToList();
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static decimal GetPrizeByCategory(int IdCategory, int IdTournament)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    return Con.DbSetCategoriesInTournament.Where(x => x.IdCategory == IdCategory && x.IdTournament == IdTournament).FirstOrDefault().Prize;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }

        }

        public static CategoriesInTournament GetByMatch(Match Match)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    return Con.DbSetDraw.Include("CategoriesInTournament").Where(x => x.IdTournament == Match.IdTournament).FirstOrDefault().CategoriesInTournament;
                }
            }
            catch (Exception e)
            {

                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }
    }
}
