﻿using System;
using System.Collections.Generic;
using System.Linq;
using TpmDAO.Entity;
using TpmDAO.Generic;
using Utils;

namespace TpmDAO.Persistence
{
    public class CategoryDAO : GenericDAO<Category, int>
    {

        public static List<Category> GetCategories()
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    return Con.DbSetCategory.Join(Con.DbSetCategoryAge, Cat => Cat.IdCategoryAge, Age => Age.IdCategoryAge, (Cat, Age) => new { Cat, Age })
                        .Join(Con.DbSetCategoryClass, Cat => Cat.Cat.IdCategoryClass, Class => Class.IdCategoryClass, (Cat, Class) => new { Cat, Class })
                        .Join(Con.DbSetCategoryGender, Cat => Cat.Cat.Cat.IdCategoryGender, Gen => Gen.IdCategoryGender, (Cat, Gen) => new { Cat, Gen }).ToList().Select(x => new Category
                        {
                            IdCategory = x.Cat.Cat.Cat.IdCategory,
                            Description = x.Gen.Description + ", " + x.Cat.Cat.Age.Description + ", " + x.Cat.Class.Class,

                        }).ToList();
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static string GetDescript(int IdCategory)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    return Con.DbSetCategory.Where(x => x.IdCategory == IdCategory).Join(Con.DbSetCategoryAge, Cat => Cat.IdCategoryAge, Age => Age.IdCategoryAge, (Cat, Age) => new { Cat, Age })
                        .Join(Con.DbSetCategoryClass, Cat => Cat.Cat.IdCategoryClass, Class => Class.IdCategoryClass, (Cat, Class) => new { Cat, Class })
                        .Join(Con.DbSetCategoryGender, Cat => Cat.Cat.Cat.IdCategoryGender, Gen => Gen.IdCategoryGender, (Cat, Gen) => new { Cat, Gen }).Select(x => x.Gen.Description + ", " + x.Cat.Cat.Age.Description + ", " + x.Cat.Class.Class).FirstOrDefault();
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static string GetDescriptWithOutGender(int IdCategory)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    return Con.DbSetCategory.Where(x => x.IdCategory == IdCategory).Join(Con.DbSetCategoryAge, Cat => Cat.IdCategoryAge, Age => Age.IdCategoryAge, (Cat, Age) => new { Cat, Age })
                        .Join(Con.DbSetCategoryClass, Cat => Cat.Cat.IdCategoryClass, Class => Class.IdCategoryClass, (Cat, Class) => new { Cat, Class })
                        .Join(Con.DbSetCategoryGender, Cat => Cat.Cat.Cat.IdCategoryGender, Gen => Gen.IdCategoryGender, (Cat, Gen) => new { Cat, Gen }).Select(x => x.Cat.Cat.Age.Description + " - " + x.Cat.Class.Class).FirstOrDefault();
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Category GetPlayerCategory(int Age, string Class, string Gender)
        {
            try
            {
                using(Connection Con = new Connection())
                {
                    return Con.DbSetCategory.Join(Con.DbSetCategoryAge, cat => cat.IdCategoryAge, age => age.IdCategoryAge, (cat, age) => new { cat, age })
                        .Join(Con.DbSetCategoryClass, cat => cat.cat.IdCategoryClass, cl => cl.IdCategoryClass, (cat, cl) => new { cat, cl })
                        .Join(Con.DbSetCategoryGender, cat => cat.cat.cat.IdCategoryGender, gen => gen.IdCategoryGender, (cat, gen) => new { cat, gen }).ToList()
                        .Select(q => new Category
                        {
                            IdCategory = q.cat.cat.cat.IdCategory,
                            IdCategoryAge = q.cat.cat.age.IdCategoryAge,
                            IdCategoryClass = q.cat.cl.IdCategoryClass,
                            IdCategoryGender = q.gen.IdCategoryGender,
                            CategoryAge = q.cat.cat.age,
                            CategoryClass = q.cat.cl,
                            CategoryGender = q.gen
                        }).ToList().Where(x => x.CategoryAge.MinAge < Age && x.CategoryAge.MaxAge >= Age && x.CategoryClass.Class.Equals(Class) && x.CategoryGender.Gender.Equals(Gender)).FirstOrDefault();
                }
            }
            catch (Exception)
            {

                throw;
            }
        }
    }
}
