﻿using System;
using System.Collections.Generic;
using System.Linq;
using TpmDAO.Entity;
using Utils;

namespace TpmDAO.Persistence
{
    public class ComboEntitiesDAO
    {
        #region Address

        public static Dictionary<string, string> ReturnDictionaryStates()
        {
            try
            {
                using (var Con = new Connection())
                {
                    List<State> States = new List<State>();
                    States.AddRange(Con.DbSetState.OrderBy(c => c.NameState).ToList()
                        .Select(a => new State()
                        {
                            IdState = a.IdState,
                            Uf = a.Uf,
                        }));
                    return States.ToDictionary(a => a.IdState.ToString(), a => a.Uf);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Dictionary<string, string> ReturnDictionaryCities(int IdState)
        {
            try
            {
                using (var Con = new Connection())
                {
                    List<City> Cities = new List<City>();
                    Cities.AddRange(Con.DbSetCity.Where(x => x.IdState == IdState).OrderBy(c => c.NameCity).ToList()
                        .Select(a => new City()
                        {
                            IdCity = a.IdCity,
                            NameCity = a.NameCity,
                        }));
                    return Cities.ToDictionary(a => a.IdCity.ToString(), a => a.NameCity);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region User

        public static Dictionary<string, string> ReturnDictionaryUsers(int IdUser = 0)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    var Users = new List<User>();
                    var oUsers = Con.DbSetUser.Include("Person").Include("UserRoles")
                        .Where(x => x.UserRoles.Where(a => a.IdRole == 2 && a.Status && !a.IsNew).FirstOrDefault() != null)
                        .OrderBy(c => c.Person.FirstName).ToList();
                    if (IdUser != 0)
                        oUsers = oUsers.Where(x => x.Person.IdPerson == IdUser).ToList();
                    Users.AddRange(oUsers
                        .Select(b => new User
                        {
                            IdUser = b.Person.IdPerson,
                            Person = new Person
                            {
                                FirstName = b.Person.FullName
                            }
                        }));

                    return Users.ToDictionary(a => a.IdUser.ToString(), a => a.Person.FirstName);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Dictionary<string, string> ReturnDictionaryHand()
        {
            try
            {
                using (var Con = new Connection())
                {
                    List<Hand> Hands = new List<Hand>();
                    Hands.AddRange(Con.DbSetHand.OrderBy(c => c.Description).ToList()
                        .Select(a => new Hand()
                        {
                            IdHand = a.IdHand,
                            Description = a.Description,
                        }));
                    return Hands.ToDictionary(a => a.IdHand.ToString(), a => a.Description);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Dictionary<string, string> ReturnDictionaryCategoryClass()
        {
            try
            {
                using (var Con = new Connection())
                {
                    List<CategoryClass> Class = new List<CategoryClass>();
                    Class.AddRange(Con.DbSetCategoryClass.OrderBy(c => c.Class).ToList()
                        .Select(a => new CategoryClass()
                        {
                            IdCategoryClass = a.IdCategoryClass,
                            Class = a.Class,
                        }));
                    return Class.ToDictionary(a => a.IdCategoryClass.ToString(), a => a.Class);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Dictionary<string, string> ReturnDictionaryUserType()
        {
            try
            {
                return new Dictionary<string, string>
                {
                    { "2", "Premium" },
                    { "3", "Básico" },
                    { "1", "Todos" }
                };
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Facilities

        public static Dictionary<string, string> ReturnDictionaryFacilities(int Owner = 0)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    var Facilities = new List<Facility>();
                    if (Owner != 0)
                        Facilities.AddRange(Con.DbSetFacility
                            .Where(x => x.Status && x.Owner == Owner)
                            );
                    else
                        Facilities.AddRange(Con.DbSetFacility
                            .Where(x => x.Status)
                            );

                    return Facilities.ToDictionary(a => a.IdFacility.ToString(), a => a.NameFacility);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Reports

        public static Dictionary<string, string> ReturnDictionaryYears()
        {
            try
            {
                var Years = Enumerable.Range(2000, DateTime.Now.Year - 1999).ToList();

                var oYears = Years.ToDictionary(x => x.ToString(), a => a.ToString());

                return oYears;
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Tournament

        public static Dictionary<string, string> ReturnDictionaryTournaments(int Owner)
        {
            try
            {
                using(Connection Con = new Connection())
                {
                    return Con.DbSetTournament.Where(x => x.Status && x.IdOwner == Owner && x.Over)
                        .ToDictionary(a => a.IdTournament.ToString(), a => a.NameTournament + " - " + a.StartDate.Date.Year);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Message

        public static Dictionary<string, string> ReturnDictionarySubjects()
        {
            try
            {
                return new Dictionary<string, string>
                {
                    { "1", "Informação" },
                    { "2", "Reclamação" },
                    { "3", "Sugestão" },
                    { "4", "Suporte" }
                };
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion
    }
}
