﻿using System;
using System.Collections.Generic;
using System.Linq;
using TpmDAO.Entity;
using TpmDAO.Entity.Misc.APIReturn;
using TpmDAO.Generic;
using Utils;

namespace TpmDAO.Persistence
{
    public class TournamentDAO : GenericDAO<Tournament, int>
    {
        public static List<Tournament> FindAllWithPlayers(int UserId = 0)
        {
            try
            {
                using(Connection Con = new Connection())
                {
                    if (UserId != 0)
                        return Con.DbSetTournament.Include("EnrolledPlayers").Where(x => x.IdOwner == UserId).ToList();
                    else
                        return Con.DbSetTournament.Include("EnrolledPlayers").ToList();
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Tournament FindComplete(int IdTournament)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    return Con.DbSetTournament.Include("EnrolledPlayers").Include("Umpires").Include("CategoriesInTournament").FirstOrDefault(x => x.IdTournament == IdTournament);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Tournament FindByIdWithPlayers(int IdTournament, int IdOwner = 0)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    if (IdOwner != 0)
                        return Con.DbSetTournament.Include("EnrolledPlayers").Where(x => x.IdTournament == IdTournament && x.IdOwner == IdOwner).FirstOrDefault();
                    else
                        return Con.DbSetTournament.Include("EnrolledPlayers").Where(x => x.IdTournament == IdTournament).FirstOrDefault();
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Tournament FindByIdWithPlayersAndCat(int IdTournament)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    return Con.DbSetTournament.Include("EnrolledPlayers").Include("CategoriesInTournament").FirstOrDefault(x => x.IdTournament == IdTournament);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static Tournament FindByIdFinish(int IdTournament)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    var Tournament = Con.DbSetTournament.Include("EnrolledPlayers.Person").Include("CategoriesInTournament").Include("Facility").Include("Owner").FirstOrDefault(x => x.IdTournament == IdTournament);

                    Tournament.CategoriesInTournament.ToList().ForEach(x => x.Draw = x.IdDraw.HasValue ? DrawDAO.FindByKeyComplete(x.IdDraw.Value) : null );

                    return Tournament;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static List<Tournament> FindByIdUmpire(int IdUmpire)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    var Tournaments = Con.DbSetTournament.Include("Umpires").Include("CategoriesInTournament").Where(x => x.Umpires.Select(p => p.IdPerson).ToList().Contains(IdUmpire) && x.CategoriesInTournament.Select(c => c.Over).ToList().Contains(false) && !x.Over).ToList();

                    foreach (var item in Tournaments)
                    {
                        item.NameFacility = FacilityDAO.FindByKey(item.IdFacility).NameFacility;
                        item.CategoriesInTournament.ToList().ForEach(x => x.Description = CategoryDAO.GetDescript(x.IdCategory));
                        item.Umpires = null;
                        if (item.StartDate <= DateTime.Now)
                        {
                            item.Started = true;
                        }
                        else
                        {
                            item.Started = false;
                        }
                    }

                    return Tournaments;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static List<Tournament> FindActiveTournaments()
        {
            try
            {
                using(Connection Con = new Connection())
                {
                    return Con.DbSetTournament.Where(x => x.EnrollmentDeadline >= DateTime.Now.Date).ToList();
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static List<Tournament> FindActiveTournamentsForUser(int IdCategory, int IdPerson)
        {
            try
            {
                using(Connection Con = new Connection())
                {
                    var Tournaments = Con.DbSetCategoriesInTournament.Include("Tournament.EnrolledPlayers")
                        .Where(x => x.IdCategory == IdCategory && (x.Tournament.EnrolledPlayers.Select(a => a.IdPerson).ToList().Contains(IdPerson) && x.Tournament.EnrollmentDeadline < DateTime.Now && x.Tournament.StartDate >= DateTime.Now) ||  x.Tournament.EnrollmentDeadline >= DateTime.Now)
                        .Select(x => x.Tournament).Distinct().ToList();

                    foreach (var item in Tournaments)
                    {
                        item.Facility = Con.DbSetFacility.Include("Address.City.State").FirstOrDefault(x => x.IdFacility == item.IdFacility);
                    }

                    return Tournaments;

                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static List<Tournament> FindActiveTournamentsForUserByCity(int IdCategory, int IdPerson, string City)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    var Tournaments = Con.DbSetCategoriesInTournament.Include("Tournament.Facility.Address.City").Include("Tournament.EnrolledPlayers")
                        .Where(x => (x.IdCategory == IdCategory && x.Tournament.Facility.Address.City.NameCity.Contains(City)) && ((x.Tournament.EnrolledPlayers.Select(a => a.IdPerson).ToList().Contains(IdPerson) && x.Tournament.EnrollmentDeadline > DateTime.Now && x.Tournament.EndDate <= DateTime.Now) ||  x.Tournament.EnrollmentDeadline >= DateTime.Now))
                        .Select(x => x.Tournament).Distinct().ToList();

                    foreach (var item in Tournaments)
                    {
                        item.Facility = Con.DbSetFacility.Include("Address.City.State").FirstOrDefault(x => x.IdFacility == item.IdFacility);
                    }

                    return Tournaments;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static List<CategoriesInTournament> FindActiveTournamentsForUserStarted(int IdPerson)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    return Con.DbSetCategoriesInTournament.Include("Tournament.EnrolledPlayers").Include("Tournament.Facility")
                        .Where(x => x.Tournament.EnrolledPlayers.Select(a => a.IdPerson).ToList().Contains(IdPerson) && x.Tournament.StartDate <= DateTime.Now).ToList();

                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        public static void UpdateOver(int IdTournament)
        {
            try
            {
                using (Connection Con = new Connection())
                {
                    var CategoriesInTournament = CategoriesInTournamentDAO.FindAllById(IdTournament);
                    if (CategoriesInTournament.Count() == CategoriesInTournament.Where(x => x.Over).Count())
                    {
                        var Tournament = TournamentDAO.FindByKey(IdTournament);
                        Tournament.Over = true;
                        TournamentDAO.Update(Tournament);
                    }
                }
            }
            catch (Exception e)
            {

                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }
    }
}
