﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using TpmMVC.Areas.RestrictedArea.Models.Adm;
using TpmMVC.Areas.RestrictedArea.Models.Premium;
using TpmMVC.Controllers;
using TpmMVC.Models;
using TpmMVC.Util;
using Utils;

namespace TpmMVC.Areas.RestrictedArea.Controllers
{
    [Authorize(Order = 0, Roles = "Admin, Premium")]
    public class HomeController : BaseController
    {
        private int UserId { get; set; } = 0;

        #region Views

        // GET: RestrictedArea/Home
        public ActionResult Index(int id = 1)
        {
            var msg = Convert.ToString(TempData["TempMessage"]) ;
            var type = Convert.ToInt32(TempData["Type"]);

            if (!string.IsNullOrEmpty(msg))
            {
                if (type == 1)
                    TempData["Success"] = msg;
                else
                    TempData["Error"] = msg;
            }
            if (id == 0)
                return View("IndexAdm");
            else
                return View();
        }

        [Authorize(Roles = "Admin")]
        [HttpGet]
        public JsonResult GetNewRequests()
        {
            try
            {
                var Return = FillRequestsGridViewModel();
                return Json(Return, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter novas requisições! - " + e.Message);
            }
        }

        [Authorize(Roles = "Admin")]
        [HttpGet]
        public ActionResult GetUserInfo(int Param)
        {
            try
            {
                FillDropDownsUser();
                var Model = FillUserViewModel(Param);
                FillDropDownCity(Convert.ToInt32(Model.State));
                return PartialView("_UserInfo", Model);
            }
            catch (Exception)
            {

                throw;
            }
        }

        #endregion

        #region Last Enrollments

        public ActionResult GetLastEnrollments()
        {
            try
            {
                var Result = FillLastEnrollmentGridView();

                return Json(Result, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter dados de inscrições! - " + e.Message);
            }
        }

        #endregion

        #region Requests

        [Authorize(Roles = "Admin")]
        [HttpGet]
        public ActionResult AcceptRequest(int IdUser)
        {
            try
            {
                var UserRole = UserRoleDAO.GetRequestByUser(IdUser);
                UserRole.Status = true;
                UserRole.IsNew = false;
                UserRoleDAO.Update(UserRole);

                var User = UserDAO.FindByKeyComplete(IdUser);

                SmtpServer.SendEmailRequest(User.Email, User.Person.FullName, 1);

                TempData["TempMessage"] = "Requisição aprovada com sucesso";
                TempData["Type"] = 1;

                return RedirectToAction("Index", new { id = 0});
            }
            catch (Exception e)
            {
                TempData["TempMessage"] = "Erro ao aprovar requisição - " + e.Message;
                TempData["Type"] = 2;
                return RedirectToAction("Index", new {id = 0 });
            }
        }

        [Authorize(Roles = "Admin")]
        [HttpGet]
        public ActionResult RefuseRequest(int IdUser)
        {
            try
            {
                var UserRole = UserRoleDAO.GetRequestByUser(IdUser);
                UserRoleDAO.Delete(UserRole);

                var User = UserDAO.FindByKeyComplete(IdUser);

                SmtpServer.SendEmailRequest(User.Email, User.Person.FullName, 2);

                TempData["TempMessage"] = "Requisição recusada com sucesso";
                TempData["Type"] = 1;
                return RedirectToAction("Index", new { id = 0 });
            }
            catch (Exception e)
            {
                TempData["TempMessage"] = "Erro ao recusar requisição - " + e.Message;
                TempData["Type"] = 2;
                return RedirectToAction("Index", new { id = 0 });
            }
        }

        #endregion

        #region Fill Grid Views

        private List<RequestsGridViewModel> FillRequestsGridViewModel()
        {
            try
            {
                var ListUserRoles = UserRoleDAO.GetNewRequests();
                var ListRequests = new List<RequestsGridViewModel>();
                if (ListUserRoles != null && ListUserRoles.Count > 0)
                {
                    foreach (var item in ListUserRoles)
                    {
                        var Request = new RequestsGridViewModel
                        {
                            IdUser = item.IdUser,
                            Name = PersonDAO.FindByKey(item.User.IdPerson).FullName,
                            Email = item.User.Email,
                            RequestDate = item.RequisitionDate.ToShortDateString(),
                            RegisterDate = UserRoleDAO.FindRolesByUser(item.User.IdUser).Where(x => x.IdRole == 3).Select(x => x.RequisitionDate).FirstOrDefault().ToShortDateString()
                        };
                        ListRequests.Add(Request);
                    }
                }
                return ListRequests;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private List<LastEnrolledPlayersGridViewModel> FillLastEnrollmentGridView()
        {
            try
            {
                var Result = EnrolledPlayersDAO.FindLastEnrollments(UserData.IdPerson);

                return Result.Select(q => new LastEnrolledPlayersGridViewModel
                {
                    IdTournament = q.IdTournament.ToString(),
                    Name = q.Person.FullName,
                    Category = CategoryDAO.GetDescript(q.IdCategory),
                    Tournament = q.Tournament.NameTournament,
                    Facility = q.Tournament.Facility.NameFacility,
                    EnrollmentDate = q.EnrollmentDate.ToString()
                }).ToList();
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Fill View Model

        private UserRegisterModel FillUserViewModel(int IdUser)
        {
            try
            {
                var User = UserDAO.FindByKeyComplete(IdUser);
                var UserViewModel = new UserRegisterModel
                {
                    Email = User.Email,
                    Name = User.Person.FirstName,
                    LastName = User.Person.LastName,
                    Gender = User.Person.Gender,
                    Tel = User.Person.Phone,
                    BirthDateFormat = User.Person.BirthDate.Value.ToString("yyyy-MM-dd"),
                    IdHand = User.Person.IdHand.ToString(),
                    IdClass = ((int)((Class)Enum.Parse(typeof(Class), User.Person.Class))).ToString(),
                    ProfilePicture = User.ProfilePicture,
                    Street = User.Person.Address.Street,
                    Number = User.Person.Address.Number,
                    Detail = User.Person.Address.Detail,
                    Neighborhood = User.Person.Address.Neighborhood,
                    Cep = User.Person.Address.ZipCode,
                    City = User.Person.Address.IdCity.ToString(),
                    State = User.Person.Address.City.IdState.ToString(),

                };

                return UserViewModel;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region FillDropDowns

        private void FillDropDownCity(int IdState)
        {
            try
            {
                ViewData["Cities"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryCities(IdState));
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion
    }
}