﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using TpmMVC.Areas.RestrictedArea.Models.Message;
using TpmMVC.Controllers;
using TpmMVC.Util;
using Utils;

namespace TpmMVC.Areas.RestrictedArea.Controllers
{
    [Authorize(Order = 0, Roles = "Admin, Premium")]
    public class MessageController : BaseController
    {

        #region View

        // GET: RestrictedArea/Message
        public ActionResult Index()
        {
            FillDropDowns();
            return View();
        }

        #endregion

        #region SendMessage

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult SendMessage(MessageViewModel Model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var Message = FillMessage(Model);

                    MessageDAO.Insert(Message);

                    TempData["Success"] = "Mensagem enviada com sucesso!";
                    return RedirectToAction("Index");
                }
                else
                {
                    FillDropDowns();
                    return View("Index", Model);
                }
            }
            catch (Exception e)
            {
                TempData["Error"] = "Ocorreu um erro ao enviar sua mensagem - " + e.Message;
                return RedirectToAction("Index", Model);
            }
        }

        private Message FillMessage(MessageViewModel Model)
        {
            try
            {
                return new Message {
                    IdUser = UserData.IdUser,
                    Subject = Enum.GetName(typeof(Subject), Convert.ToInt32(Model.Subject)),
                    Date = DateTime.Now,
                    MessageBody = Model.MessageBody
            };
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Messages

        [HttpGet]
        public JsonResult GetMessages()
        {
            try
            {
                var Messages = FillMessages();

                return Json(Messages, JsonRequestBehavior.AllowGet);

            }
            catch (Exception e)
            {
                return GetError("Erro ao carregar mensagens! - " + e.Message);
            }
        }

        [HttpGet]
        public JsonResult SetMessageRead(int Param)
        {
            try
            {
                var Message = MessageDAO.FindByKey(Param);
                Message.IsRead = true;
                Message.ReadDate = DateTime.Now;
                MessageDAO.Update(Message);

                return GetSuccess("Mensagem marcada como lida!");
            }
            catch (Exception e)
            {
                return GetError("Erro ao marcar mensagem como lida! - " + e.Message);
            }
        }

        [HttpGet]
        public ActionResult ReadMessage(int Param)
        {
            try
            {
                var Message = FillMessage(Param);

                return View("_ReadMessage", Message);

            }
            catch (Exception e)
            {
                return GetError("Erro ao ler mensagem! - " + e.Message);
            }
        }


        private List<MessageGridViewModel> FillMessages()
        {
            try
            {
                var Messages = MessageDAO.GetMessages();

                return Messages.Select(q => new MessageGridViewModel
                {
                    IdMessage = q.IdMessage,
                    IsRead = q.IsRead,
                    Subject = q.Subject,
                    Date = q.Date.ToString("dd/MM/yyyy HH:mm"),
                    MessageBody = q.MessageBody,
                    User = q.User.Person.FullName,
                    IdUser = q.User.IdUser
                }).ToList();
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private MessageGridViewModel FillMessage(int IdMessage)
        {
            try
            {
                var Message = MessageDAO.FindByKeyComplete(IdMessage);
                Message.IsRead = true;
                Message.ReadDate = DateTime.Now;
                MessageDAO.Update(Message);

                return new MessageGridViewModel
                {
                    IdMessage = Message.IdMessage,
                    Date = Message.Date.ToString("dd/MM/yyyy HH:mm"),
                    IdUser = Message.IdUser,
                    MessageBody = Message.MessageBody,
                    Subject = Message.Subject,
                    User = Message.User.Person.FullName,
                    IsRead = Message.IsRead
                };
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Response

        [HttpPost]
        public ActionResult ResponseMessage(string MessageResponse, int IdMessage)
        {
            try
            {
                var Message = MessageDAO.FindByKeyComplete(IdMessage);

                var User = Message.User.Person.FullName;
                var Date = Message.Date;
                var Subject = Message.Subject;
                var MessageContent = Message.MessageBody;

                SmtpServer.SendEmailResponse(User, Date, Subject, MessageContent, MessageResponse, Message.User.Email);

                return GetSuccess("Mensagem enviada com sucesso!");
            }
            catch (Exception e)
            {
                return GetError("Erro ao responder mensagem! - " + e.Message);
            }
        }

        #endregion

        #region FillDropDowns

        private void FillDropDowns()
        {
            try
            {
                ViewData["Subjects"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionarySubjects());
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion
    }
}