﻿using CrystalDecisions.Shared;
using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using TpmMVC.Areas.RestrictedArea.Models.Report;
using TpmMVC.Areas.RestrictedArea.Reports.CrystalFiles;
using TpmMVC.Areas.RestrictedArea.Reports.DataSets;
using TpmMVC.Controllers;
using TpmMVC.Util;
using Utils;

namespace TpmMVC.Areas.RestrictedArea.Controllers
{
    [Authorize]
    public class ReportController : BaseController
    {
        private int UserId { get; set; } = 0;

        #region View
        [Authorize(Order = 0, Roles = "Admin, Premium")]
        // GET: RestrictedArea/Report
        public ActionResult Index()
        {
            try
            {
                FillDropDownReport();
                FillDropDownsUser();
                return View();
            }
            catch (Exception e)
            {
                TempData["TempMessage"] = "Erro ao carregar opções de combobox - " + e.Message;
                TempData["Type"] = 0;
                return RedirectToAction("Index", "Home");
            }
        }

        #endregion

        #region Request ADM

        [Authorize(Roles = ("Admin"))]
        [HttpPost]
        public ActionResult ReportUsers(ReportViewModel Model)
        {
            try
            {

                var Report = FillReportUsers(Model);

                return File(Report, "application/pdf");

            }
            catch (Exception e)
            {
                TempData["TempMessage"] = "Erro ao gerar relatório - " + e.Message;
                TempData["Type"] = 0;
                return RedirectToAction("Index", "Home");
            }
        }

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpPost]
        public ActionResult ReportFacilities(ReportViewModel Model)
        {
            try
            {

                var Report = FillReportFacilities(Model);

                return File(Report, "application/pdf");
            }
            catch (Exception e)
            {
                TempData["TempMessage"] = "Erro ao gerar relatório - " + e.Message;
                TempData["Type"] = 0;
                return RedirectToAction("Index", "Home");
            }
        }

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpPost]
        public ActionResult ReportTournament(ReportViewModel Model)
        {
            try
            {
                var Report = FillReportTournaments(Model);

                return File(Report, "application/pdf");
            }
            catch (Exception e)
            {
                TempData["TempMessage"] = "Erro ao gerar relatório - " + e.Message;
                TempData["Type"] = 0;
                return RedirectToAction("Index", "Home");
            }
        }

        [Authorize(Roles = ("Admin"))]
        [HttpPost]
        public ActionResult ReportPremium(ReportViewModel Model)
        {
            try
            {

                var Report = FillReportPremium(Model);


                return File(Report, "application/pdf");
            }
            catch (Exception e)
            {
                TempData["TempMessage"] = "Erro ao gerar relatório - " + e.Message;
                TempData["Type"] = 0;
                return RedirectToAction("Index", "Home", new { id = 0 });
            }
        }

        [Authorize(Roles = ("Premium"))]
        [HttpPost]
        public ActionResult ReportTournamentDetail(ReportViewModel Model)
        {
            try
            {

                var Report = FillTournamentDetails(Model);


                return File(Report, "application/pdf");
            }
            catch (Exception e)
            {
                TempData["TempMessage"] = "Erro ao gerar relatório - " + e.Message;
                TempData["Type"] = 0;
                return RedirectToAction("Index", "Home");
            }
        }

        #endregion

        #region Request Premium


        #endregion

        #region Fill Reports

        private Stream FillReportUsers(ReportViewModel Model)
        {
            try
            {
                var Users = ReportDAO.ReportUsers(Model.StartDate, Model.EndDate, Model.UserType);

                if (Model.UserType == 3)
                    Users = Users.Where(x => x.UserRoles.Where(a => a.IdRole == 3).FirstOrDefault() != null && x.UserRoles.Where(a => a.IdRole == 2).FirstOrDefault() == null).ToList();

                var DSUser = new DSUsers();
                var Dt = DSUser.Tables["Users"];

                foreach (var item in Users)
                {
                    var Role = item.UserRoles.Where(x => x.IdRole == 2).FirstOrDefault() != null ? item.UserRoles.Where(x => x.IdRole == 2).FirstOrDefault() : item.UserRoles.FirstOrDefault(x => x.IdRole == 3);
                    var Row = Dt.NewRow();
                    Row["IdPerson"] = item.IdPerson;
                    Row["FullName"] = item.Person.FullName;
                    Row["Email"] = item.Email;
                    Row["Tel"] = SetMasks(item.Person.Phone);
                    Row["Address"] = item.Person.Address.ToString();
                    Row["Type"] = item.UserRoles.Any(x => x.IdRole == 2) ? "Premium" : "Básico";
                    Row["RegisterDate"] = Role.RequisitionDate;
                    Row["Status"] = Role.Status;
                    Dt.Rows.Add(Row);

                }

                var UserReport = new UsersReport();
                UserReport.SetDataSource(DSUser);

                return UserReport.ExportToStream(ExportFormatType.PortableDocFormat);

            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private Stream FillReportFacilities(ReportViewModel Model)
        {
            try
            {
                if (User.IsInRole("Premium"))
                    UserId = UserData.IdPerson;

                var Facilities = ReportDAO.ReportFacilities(UserId, Model.State, Model.City);

                var DSFacilities = new DSFacilities();
                var Dt = DSFacilities.Tables["Facilities"];

                foreach (var item in Facilities)
                {
                    var Row = Dt.NewRow();
                    Row["IdFacility"] = item.IdFacility;
                    Row["Name"] = item.NameFacility;
                    Row["Email"] = item.Email;
                    Row["Tel"] = SetMasks(item.Phone);
                    Row["Address"] = item.Address.ToString();
                    Row["Owner"] = PersonDAO.FindByKey(item.Owner).FullName;
                    Row["Status"] = item.Status;
                    Dt.Rows.Add(Row);

                }

                var FacilityReport = new FacilityReport();
                FacilityReport.SetDataSource(DSFacilities);

                return FacilityReport.ExportToStream(ExportFormatType.PortableDocFormat);

            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private Stream FillReportTournaments(ReportViewModel Model)
        {
            try
            {
                if (User.IsInRole("Premium"))
                    UserId = UserData.IdPerson;

                var Tournaments = ReportDAO.ReportTournaments(UserId, Model.StartDate, Model.EndDate);

                var DSTournament = new DSTournament();
                var Dt = DSTournament.Tables["Tournaments"];

                foreach (var item in Tournaments)
                {
                    var Row = Dt.NewRow();
                    Row["IdTournament"] = item.IdTournament;
                    Row["Name"] = item.NameTournament;
                    Row["StartDate"] = item.StartDate;
                    Row["EndDate"] = item.EndDate;
                    Row["Facility"] = item.Facility.NameFacility;
                    Row["Owner"] = PersonDAO.FindByKey(item.Facility.Owner).FullName;
                    Row["EnrolledPlayers"] = item.EnrolledPlayers.Count;
                    Dt.Rows.Add(Row);
                    Row["Status"] = item.Status;

                }

                var TournamentReport = new TournamentReport();

                TournamentReport.SetDataSource(DSTournament);

                return TournamentReport.ExportToStream(ExportFormatType.PortableDocFormat);
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private Stream FillReportPremium(ReportViewModel Model)
        {
            try
            {
                var UsersPremium = ReportDAO.ReportPremiumUsers(Convert.ToInt32(Model.Year));

                var DSPremium = new DSPremium();
                var Dt = DSPremium.Tables["Premium"];

                foreach (var item in UsersPremium)
                {
                    var Row = Dt.NewRow();
                    Row["IdUser"] = item.IdUser;
                    Row["FullName"] = item.Person.FullName;
                    Row["Email"] = item.Email;
                    Row["RequisitionDate"] = item.UserRoles.FirstOrDefault().RequisitionDate;
                    Row["Month"] = item.UserRoles.FirstOrDefault().RequisitionDate.Month;
                    Row["Facebook"] = !string.IsNullOrEmpty(item.FacebookId) ? true : false;
                    Row["Year"] = Model.Year;
                    Row["Status"] = item.UserRoles.FirstOrDefault().Status;
                    Row["MonthName"] = DateTimeFormatInfo.CurrentInfo.GetMonthName(item.UserRoles.FirstOrDefault().RequisitionDate.Month).ToUpper();
                    Dt.Rows.Add(Row);

                }

                var PremiumReport = new PremiumReport();

                PremiumReport.SetDataSource(DSPremium);

                return PremiumReport.ExportToStream(ExportFormatType.PortableDocFormat);
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private Stream FillTournamentDetails(ReportViewModel Model)
        {
            try
            {

                var Tournament = TournamentDAO.FindByIdFinish(Convert.ToInt32(Model.Tournament));

                var DSDetail = new DSTournamentDetail();

                var DtTournament = DSDetail.Tables["Tournament"];
                var DtTCategory = DSDetail.Tables["Category"];
                var DtPlayers = DSDetail.Tables["Players"];

                #region Fill Tournament

                var Row = DtTournament.NewRow();
                Row["IdTournament"] = Tournament.IdTournament;
                Row["NameTournament"] = Tournament.NameTournament;
                Row["StartDate"] = Tournament.StartDate;
                Row["EndDate"] = Tournament.EndDate;
                Row["Facility"] = Tournament.Facility.NameFacility;
                Row["Owner"] = Tournament.Owner.FullName;
                Row["Enrolled"] = Tournament.EnrolledPlayers.Count;
                DtTournament.Rows.Add(Row);

                FillCategories(ref Tournament, ref DtTCategory);

                FillPlayers(ref Tournament, ref DtPlayers);

                #endregion

                var DetailReport = new TournamentDetail();

                DetailReport.SetDataSource(DSDetail);

                return DetailReport.ExportToStream(ExportFormatType.PortableDocFormat);
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private void FillCategories(ref Tournament Tournament, ref DataTable DtCategories)
        {
            try
            {
                foreach (var item in Tournament.CategoriesInTournament)
                {
                    var Row = DtCategories.NewRow();
                    Row["IdCategory"] = item.IdCategory;
                    Row["Descript"] = CategoryDAO.GetDescript(item.IdCategory);
                    Row["IdTournament"] = item.IdTournament;
                    Row["Prize"] = item.Prize;
                    Row["PlayersCount"] = Tournament.EnrolledPlayers.Where(x => x.IdCategory == item.IdCategory).ToList().Count;
                    DtCategories.Rows.Add(Row);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private void FillPlayers(ref Tournament Tournament, ref DataTable DtPlayers)
        {
            try
            {
                foreach (var item in Tournament.EnrolledPlayers)
                {
                    var Draw = Tournament.CategoriesInTournament.Where(x => x.IdCategory == item.IdCategory).FirstOrDefault().Draw;
                    var LastMatch = Draw.Matches.Where(x => x.MatchStats.Where(a => a.IdPerson == item.IdPerson && !string.IsNullOrEmpty(a.Result)).FirstOrDefault() != null).LastOrDefault();
                    var Opponent = LastMatch.MatchStats.Where(a => a.IdPerson != item.IdPerson).FirstOrDefault();

                    var Row = DtPlayers.NewRow();
                    Row["IdCategory"] = item.IdCategory;
                    Row["Name"] = item.Person.FullName;
                    Row["IdPerson"] = item.IdPerson;
                    Row["EnrolledDate"] = item.EnrollmentDate;
                    Row["Win"] = Draw.Matches.Where(x => x.MatchStats.Where(a => a.IdPerson == item.IdPerson && !string.IsNullOrEmpty(a.Result) && a.Result.Equals("W")).FirstOrDefault() != null).ToList().Count;
                    Row["Lost"] = Draw.Matches.Where(x => x.MatchStats.Where(a => a.IdPerson == item.IdPerson && !string.IsNullOrEmpty(a.Result) && a.Result.Equals("L")).FirstOrDefault() != null).ToList().Count;
                    Row["IdRound"] = LastMatch.IdRound;
                    Row["Round"] = LastMatch.Round.NameRound;
                    Row["Opponent"] = Opponent != null ? Opponent.Person.FullName : "BYE";
                    Row["Winner"] = Draw.Matches.Where(x => x.IdRound == 1).FirstOrDefault().MatchStats.Any(a => a.IdPerson == item.IdPerson && !string.IsNullOrEmpty(a.Result) && a.Result.Equals("W"));
                    DtPlayers.Rows.Add(Row);
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Masks

        private string SetMasks(string Value, string Type = "")
        {
            try
            {
                    return "(" + new string(Value.Take(2).ToArray()) + ") " + new string(Value.Skip(2).ToArray()); 
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Fill DropDowns

        private void FillDropDownReport()
        {
            try
            {
                ViewData["UserTypes"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryUserType());
                ViewData["Years"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryYears());
                if (User.IsInRole("Premium"))
                    ViewData["Tournaments"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryTournaments(UserData.IdPerson));
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion
    }
}