﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using TpmDAO.Entity;
using TpmDAO.Persistence;
using TpmMVC.Areas.RestrictedArea.Models.Tournament;
using TpmMVC.Controllers;
using System.Linq;
using System.Threading.Tasks;
using Newtonsoft.Json;
using TpmMVC.Util;
using Utils;
using System.Globalization;

namespace TpmMVC.Areas.RestrictedArea.Controllers
{

    public class TournamentController : BaseController
    {
        private int UserId { get; set; } = 0;

        #region Views

        // GET: RestrictedArea/Tournament
        [Authorize(Order = 0, Roles = "Admin, Premium")]
        public ActionResult Index(int id = 0)
        {
            ViewData["IdTournamentParam"] = id;
            FillDropDown();
            return View();
        }

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public ActionResult Create()
        {
            FillDropDown();
            return PartialView("Create");
        }

        #endregion

        #region Save

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpPost]
        public async Task<ActionResult> SaveTournamentAsync(TournamentViewModel Model, string Categories, string Umpires, string Players)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    Model.TournamentCategories = JsonConvert.DeserializeObject<List<CategoryGridViewModel>>(Categories);

                    Model.TournamentUmpires = JsonConvert.DeserializeObject<List<UmpireGridViewModel>>(Umpires);

                    var Tournament = await FillTournamentAsync(Model);

                    var OldTournament = new Tournament();
                    var Update = false;
                    if (Model.IdTournament != 0)
                    {
                        var EnrolledPlayers = JsonConvert.DeserializeObject<List<EnrolledPlayersGridViewModel>>(Players);

                        foreach (var item in EnrolledPlayers)
                        {
                            if (!Model.TournamentCategories.Select(x => x.IdCategory.ToString()).ToList().Contains(item.IdCategory))
                            {
                                var Player = EnrolledPlayersDAO.FindPlayerEnrolled(Convert.ToInt32(item.IdPerson), Model.IdTournament);
                                EnrolledPlayersDAO.Delete(Player);
                            }
                        }

                        OldTournament = TournamentDAO.FindComplete(Model.IdTournament);
                        if (Model.TournamentPicture != null && !string.IsNullOrEmpty(OldTournament.TournamentPicture))
                            await Images.DeletePicture(OldTournament.TournamentPicture, "tournament");

                        UpdateObjectChildren(ref Tournament, ref OldTournament);

                        Update = true;
                    }

                    if (Update)
                        TournamentDAO.Update(Tournament);
                    else
                        TournamentDAO.Insert(Tournament);

                    return GetSuccess("Torneio salvo com sucesso!");
                }
                else
                {
                    FillDropDown();
                    Model.CategoriesJson = Categories;
                    Model.UmpiresJson = Umpires;
                   
                    if (Model.IdTournament != 0)
                    {
                         var ErrorMessages = "";
                        var Errors = ModelState.Values.SelectMany(v => v.Errors.Select(b => b.ErrorMessage)).ToList();
                       
                        Errors.ForEach(a => ErrorMessages += a + "<br />");

                        return GetError(ErrorMessages);
                }
                    else
                        return PartialView("Create", Model);
                }
            }
            catch (Exception e)
            {
                return GetError("Erro ao salvar torneio! - " + e.Message);
            }
        }

        #endregion

        #region Matches Dates

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpPost]
        public JsonResult SetMachesTime(string oMatches)
        {
            try
            {
                if (!string.IsNullOrEmpty(oMatches))
                {
                    var Matches = JsonConvert.DeserializeObject<List<MatchesGridViewModel>>(oMatches).Where(x => x.IsUpdate).ToList();

                    foreach (var item in Matches)
                    {
                        var oMatch = MatchDAO.FindByKey(item.IdMatch);

                        if (!string.IsNullOrEmpty(item.Date) && !string.IsNullOrEmpty(item.Time))
                            oMatch.DateMatch = Convert.ToDateTime(item.Date).Add(TimeSpan.Parse(item.Time));
                        MatchDAO.Update(oMatch);
                    }

                    return GetSuccess("Alterações salvas com sucesso!");
                }
                else
                    return GetWarning("Nenhuma alteração salva!");
            }
            catch (Exception e)
            {
                return GetError("Erro ao atualizar Partidas - " + e.Message);
            }
        }

        #endregion

        #region Lists

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public JsonResult ListTournaments()
        {
            try
            {
                var Tournaments = FillGridViewTournament();
                return Json(Tournaments, JsonRequestBehavior.AllowGet);

            }
            catch (Exception e)
            {
                return GetError("Erro ao listar torneios! - " + e.Message);
            }
        }

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public JsonResult ListCategories()
        {
            try
            {
                var Categories = FillGridViewCategories();
                return Json(Categories, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter categorias! - " + e.Message);
            }
        }

        #endregion

        #region Update Childs

        private void UpdateObjectChildren(ref Tournament Tournament, ref Tournament OldTournament)
        {
            try
            {
                foreach (var New in Tournament.Umpires.ToList())
                {
                    if (!OldTournament.Umpires.Select(x => x.IdPerson).Contains(New.IdPerson))
                        UmpireDAO.Insert(New);

                }

                foreach (var Old in OldTournament.Umpires.ToList())
                {
                    if (!Tournament.Umpires.Select(x => x.IdPerson).Contains(Old.IdPerson))
                        UmpireDAO.Delete(Old);
                }

                foreach (var old in OldTournament.CategoriesInTournament.ToList())
                {
                    if (!Tournament.CategoriesInTournament.Select(x => x.IdCategory).Contains(old.IdCategory))
                        CategoriesInTournamentDAO.Delete(old);
                }

                foreach (var New in Tournament.CategoriesInTournament.ToList())
                {
                    if (!OldTournament.CategoriesInTournament.Select(x => x.IdCategory).Contains(New.IdCategory))
                        CategoriesInTournamentDAO.Insert(New);
                    else
                        CategoriesInTournamentDAO.Update(New);
                }
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Remove Player

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public JsonResult RemovePlayer(string Param)
        {
            try
            {
                int IdPerson = Convert.ToInt32(Param.Split(',').ElementAt(0));
                int IdTournament = Convert.ToInt32(Param.Split(',').ElementAt(1));

                var Player = EnrolledPlayersDAO.FindPlayerEnrolled(IdPerson, IdTournament);
                EnrolledPlayersDAO.Delete(Player);

                return GetSuccess("Jogador removido com sucesso!");

            }
            catch (Exception e)
            {
                return GetError("Erro ao remover jogardor! - " + e.Message);
            }
        }

        #endregion

        #region GetJsonData

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public JsonResult GetTournamentInfo(int Param)
        {
            try
            {
                var Tournament = FillViewModelTournament(Param);
                return Json(Tournament, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter dados do torneio! - " + e.Message);
            }
        }

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public JsonResult ListCategoriesTournament(int Param)
        {
            try
            {
                var Categories = FillCatGridViewByTournament(Param);

                return Json(Categories, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public JsonResult GetUmpires(int Param)
        {
            try
            {
                var Umpires = FillUmpireGridViewModel(Param);

                return Json(Umpires, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao listar Árbitros! - " + e.Message);
            }
        }

        /// <summary>
        /// Get info for an user
        /// </summary>
        /// <param name="Param">User ID</param>
        /// <returns>Json User object </returns>
        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public JsonResult GetUserInfo(string Param)
        {
            try
            {
                var User = FillViewUserInfo(Param);
                return Json(User, JsonRequestBehavior.AllowGet);
            }
            catch (Exception e)
            {
                return GetError("Erro ao obter dados do usuário! - " + e.Message);
            }
        }

        #endregion

        #region Active/Deactive

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public ActionResult RemoveTournament(int IdTournament)
        {
            try
            {
                var Tournament = TournamentDAO.FindByKey(IdTournament);
                Tournament.Status = false;
                TournamentDAO.Update(Tournament);
                TempData["Success"] = "Torneio desativado com sucesso!";
                return RedirectToAction("Index");
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao desativar torneio! - " + e.Message;
                return RedirectToAction("Index");
            }
        }

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public ActionResult ReactiveTournament(int IdTournament)
        {
            try
            {
                var Tournament = TournamentDAO.FindByKey(IdTournament);
                Tournament.Status = true;
                TournamentDAO.Update(Tournament);
                TempData["Success"] = "Torneio reativado com sucesso!";
                return RedirectToAction("Index");
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao reativar torneio! - " + e.Message;
                return RedirectToAction("Index");
            }
        }

        #endregion

        #region Fill Objects

        /// <summary>
        /// Fill Tournament object with Model data
        /// </summary>
        /// <param name="Model">Tournament ViewModel with form data</param>
        /// <param name="List">List of Categories json deserialized</param>
        /// <returns>Tournament object</returns>
        private async Task<Tournament> FillTournamentAsync(TournamentViewModel Model)
        {
            try
            {
                var Tournament = new Tournament
                {
                    Description = Model.Description,
                    EndDate = Model.EndDate,
                    IdFacility = Convert.ToInt32(Model.IdFaciity),
                    IdOwner = Convert.ToInt32(Model.IdOwner),
                    NameTournament = Model.NameTournament,
                    IdTournament = Model.IdTournament,
                    StartDate = Model.StartDate,
                    EnrollmentDeadline = Model.EnrollmentDeadline,
                    EnrollmentPrice = Model.EnrollmentPrice,
                    CategoriesInTournament = Model.TournamentCategories.Select(x => new CategoriesInTournament
                    {
                        IdCategory = x.IdCategory,
                        Prize = x.Prize,
                        Draw = null,
                        IdTournament = Model.IdTournament,
                        Tournament = null
                    }).ToList(),
                    Umpires = Model.TournamentUmpires.Select(x => new Umpire
                    {
                        IdPerson = x.IdPerson,
                        IdTournament = x.IdTournament,
                    }).ToList()
                };

                if (Model.TournamentPicture != null)
                    Tournament.TournamentPicture = await Images.UploadPicture(Model.TournamentPicture, Helpers.GenerateFileName(Model.TournamentPicture.FileName), "tournament");

                return Tournament;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Fill GridViews

        private List<TournamentGridViewModel> FillGridViewTournament()
        {
            try
            {
                if (User.IsInRole("Premium"))
                    UserId = UserData.Person.IdPerson;
                var Tournaments = TournamentDAO.FindAllWithPlayers(UserId);

                var TournamentsVM = Tournaments.Select(x => new TournamentGridViewModel
                {
                    IdTournament = x.IdTournament,
                    StartDate = x.StartDate.ToShortDateString(),
                    EnrollmentDeadLine = x.EnrollmentDeadline.ToShortDateString(),
                    Name = x.NameTournament,
                    EnrolledPlayers = x.EnrolledPlayers.Count(),
                    Status = x.Status,
                    HasDraw = DrawDAO.FindByTournament(x.IdTournament) != null ? true : false,
                    HasPast = x.Over,
                    CanDraw = !x.Over && x.EnrollmentDeadline.Date < DateTime.Now.Date && x.EnrolledPlayers.Count() > 0,
                    EndEnrollment = x.EnrollmentDeadline.Date < DateTime.Now.Date
                }).OrderByDescending(x => x.StartDate).ToList();
                return TournamentsVM;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private List<CategoryGridViewModel> FillGridViewCategories()
        {
            var Categories = CategoryDAO.GetCategories();

            var CategoriesVM = Categories.Select(x => new CategoryGridViewModel
            {
                IdCategory = x.IdCategory,
                Description = x.Description,
                Prize = 0,
                Active = false
            }).ToList();

            return CategoriesVM;
        }

        private List<CategoryGridViewModel> FillCatGridViewByTournament(int Param)
        {
            try
            {
                var Categories = CategoriesInTournamentDAO.FindAllById(Param);

                var CategoriesVM = Categories.Select(x => new CategoryGridViewModel
                {
                    IdCategory = x.IdCategory,
                    Description = x.Description,
                    Prize = x.Prize,
                    Active = true,
                    Key = x.IdDraw ?? 0
                }).ToList();

                return CategoriesVM;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private TournamentViewModel FillViewModelTournament(int IdTournament)
        {
            try
            {
                if (User.IsInRole("Premium"))
                    UserId = UserData.Person.IdPerson;

                var Tournament = TournamentDAO.FindByIdWithPlayers(IdTournament, UserId);

                if (Tournament != null)
                {
                    var TournamentVM = new TournamentViewModel
                    {
                        Description = Tournament.Description,
                        EndDate = Tournament.EndDate,
                        EnrollmentDeadline = Tournament.EnrollmentDeadline,
                        EnrollmentPrice = Tournament.EnrollmentPrice,
                        IdFaciity = Tournament.IdFacility.ToString(),
                        IdOwner = Tournament.IdOwner.ToString(),
                        IdTournament = Tournament.IdTournament,
                        NameTournament = Tournament.NameTournament,
                        Picture = Tournament.TournamentPicture,
                        StartDate = Tournament.StartDate,
                    };
                    var Players = Tournament.EnrolledPlayers.Select(x => new EnrolledPlayersGridViewModel
                    {
                        IdPerson = x.IdPerson.ToString(),
                        FullName = PersonDAO.FindByKey(x.IdPerson).FullName,
                        EnrollmentDate = x.EnrollmentDate.ToString(),
                        Category = CategoryDAO.GetDescript(x.IdCategory),
                        IdCategory = x.IdCategory.ToString()
                    }).ToList();

                    TournamentVM.TournamentPlayers = Players;
                    return TournamentVM;
                }
                else
                    return null;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private List<UmpireGridViewModel> FillUmpireGridViewModel(int IdTournament)
        {
            try
            {
                var Umpires = UmpireDAO.FindByTournament(IdTournament);
                return Umpires.Select(x => new UmpireGridViewModel
                {
                    Email = x.Email,
                    IdPerson = x.IdPerson,
                    IdTournament = x.IdTournament,
                    Name = x.Name
                }).ToList();
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private List<MatchesGridViewModel> FillMatchesGridViewModel(string Param)
        {
            try
            {
                var IdTournament = Convert.ToInt32(Param.Split(',')[0]);
                var IdCategory = Convert.ToInt32(Param.Split(',')[1]);

                var Draw = DrawDAO.FindByTournamentAndCategory(IdTournament, IdCategory);

                var Matches = Draw.Matches.Where(x => !x.ByeMatch).ToList();

                var MatchesVM = Matches.Select(q => new MatchesGridViewModel
                {
                    IdMatch = q.IdMatch,
                    Player1 = q.MatchStats.Count > 0 ? q.MatchStats.FirstOrDefault().Person.FullName : "Aguardando resultado",
                    Player2 = q.MatchStats.Count > 1 ? q.MatchStats.ElementAt(1).Person.FullName : "Aguardando resultado",
                    Round = q.Round.NameRound,
                    Date = q.DateMatch.HasValue ? q.DateMatch.Value.ToShortDateString() : "",
                    Time = q.DateMatch.HasValue ? q.DateMatch.Value.TimeOfDay.ToString() : ""
                }).ToList();

                return MatchesVM;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Matches

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public ActionResult GetMatchesFromTournament(string Param)
        {
            try
            {

                var Result = FillMatchesGridViewModel(Param);

                return Json(Result, JsonRequestBehavior.AllowGet);

            }
            catch (Exception e)
            {
                return GetError("Erro ao obter partidas! - " + e.Message);
            }
        }

        #endregion

        #region Fill DropDown

        private void FillDropDown()
        {
            try
            {
                FillDropDownsUser();
                if (User.IsInRole("Premium"))
                    UserId = UserData.Person.IdPerson;

                ViewData["Owner"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryUsers(UserId));
                ViewData["Facilities"] = Helpers.FillDropDown(ComboEntitiesDAO.ReturnDictionaryFacilities(UserId));

            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Generate Draw

        [Authorize(Order = 0, Roles = "Admin, Premium")]
        [HttpGet]
        public ActionResult GenerateDraw(int IdTournament)
        {
            try
            {
                var Tournament = TournamentDAO.FindByIdWithPlayersAndCat(IdTournament);
                GenerateRandomMatch(ref Tournament);

                TempData["Success"] = "Chave de torneio gerada com sucesso!";
                return RedirectToAction("Index");
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao gerar chave de torneio! -" + e.Message;
                return RedirectToAction("Index");
            }
        }

        private void GenerateRandomMatch(ref Tournament Tournament)
        {
            try
            {
                foreach (var item in Tournament.CategoriesInTournament.ToList())
                {
                    var Players = Tournament.EnrolledPlayers.Where(x => x.IdCategory == item.IdCategory).OrderBy(x => Guid.NewGuid()).ToList();
                    if (Players.Count >= 4)
                    {
                        var Draw = new Draw
                        {
                            IdTournament = item.IdTournament,
                            IdCategory = item.IdCategory,
                            Matches = SortPlayers(ref Players, item.IdTournament, SetRoundType(0, Players.Count, true))
                        };
                        DrawDAO.Insert(Draw);
                        item.IdDraw = Draw.IdDraw;
                        CategoriesInTournamentDAO.Update(item);

                        var Count = 1;
                        foreach (var Match in Draw.Matches)
                        {
                            if (Count != Draw.Matches.Count)
                            {
                                var NMacth = Draw.Matches.Where(x => x.Sequence == Match.NextMatchTemp).FirstOrDefault();
                                Match.IdNextMatch = NMacth.IdMatch;
                                if (Match.MatchStats != null && Match.MatchStats.Count == 1)
                                {
                                    var Stats = new Stats
                                    {
                                        IdMatch = NMacth.IdMatch,
                                        IdPerson = Match.MatchStats.FirstOrDefault().IdPerson
                                    };
                                    StatsDAO.Insert(Stats);
                                }
                                MatchDAO.Update(Match);
                                Count++;
                            }
                        }
                    }
                    else
                    {
                        foreach (var Player in Players)
                        {
                            EnrolledPlayersDAO.Delete(Player);
                        }
                        CategoriesInTournamentDAO.Delete(item);
                    }
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private List<Match> SortPlayers(ref List<EnrolledPlayers> Players, int IdTournament, int IdRound)
        {
            try
            {
                var PlayersTotal = Players.Count;
                var Matches = new List<Match>();
                var HasBye = IsByeRound(PlayersTotal);
                var CountTotal = GetRoundsTotal(PlayersTotal, IdRound);

                var TotalRounds = GetRoundsTotal(PlayersTotal, 0);
                //Matches for the first Round with byes
                var CountWithByes = 0;
                if (HasBye)
                    CountWithByes = ByeCountFirstRound(PlayersTotal);

                var CountAux = PlayersTotal;

                var CountMatches = CountTotal;
                var AuxCountMatches = CountMatches;

                var Array = MatchesBye(CountMatches, CountWithByes);
                var Sequence = 1;

                while (CountMatches > 0)
                {

                    var Match = new Match
                    {
                        IdRound = IdRound,
                        IdTournament = IdTournament,
                        Sequence = Sequence,
                        MatchStats = new List<Stats>(),
                    };

                    if (Array[Sequence - 1])
                    {
                        var Stats = new Stats
                        {
                            IdPerson = Players.ElementAt(CountAux - 1).IdPerson,
                            Result = "W"
                        };
                        Match.ByeMatch = true;
                        Match.Over = true;
                        Match.MatchStats.Add(Stats);
                        CountAux--;

                        Matches.Add(Match);
                        CountMatches--;
                        Sequence++;
                    }
                    else
                    {
                        var StatsP1 = new Stats
                        {
                            IdPerson = Players.ElementAt(CountAux - 1).IdPerson
                        };
                        Match.MatchStats.Add(StatsP1);
                        CountAux--;

                        var StatsP2 = new Stats
                        {
                            IdPerson = Players.ElementAt(CountAux - 1).IdPerson
                        };
                        Match.MatchStats.Add(StatsP2);
                        CountAux--;

                        Matches.Add(Match);
                        CountMatches--;
                        Sequence++;
                    }
                }

                TotalRounds--;
                var CurrentRound = IdRound;
                while (TotalRounds > 0)
                {
                    CurrentRound = SetRoundType(CurrentRound, PlayersTotal, false);
                    var oMatchesByRound = GetRoundsTotal(PlayersTotal, CurrentRound);
                    while (oMatchesByRound > 0)
                    {
                        var oMatch = new Match
                        {
                            Sequence = Sequence,
                            IdRound = CurrentRound,
                            IdTournament = IdTournament
                        };
                        Matches.Add(oMatch);
                        oMatchesByRound--;
                        Sequence++;
                    }
                    TotalRounds--;
                }

                SetNextRound(AuxCountMatches, ref Matches);

                return Matches;
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private bool[] MatchesBye(int CountMatches, int CountByes)
        {
            try
            {
                var Random = new Random();
                var Array = new bool[CountMatches];
                var Aux = 0;

                while (CountByes > 0)
                {
                    switch (Aux)
                    {
                        case 0:
                            Array[0] = true;
                            CountByes--;
                            break;
                        case 1:
                            Array[CountMatches - 1] = true;
                            CountByes--;
                            break;
                        case 2:
                            Array[CountMatches / 2 - 1] = true;
                            CountByes--;
                            break;
                        case 3:
                            Array[CountMatches / 2] = true;
                            CountByes--;
                            break;
                        default:
                            if (CountByes % 2 == 0)
                            {
                                var Num = Random.Next(0, CountMatches / 2 - 1);

                                while (!Array[Num])
                                {
                                    Array[Num] = true;
                                    CountByes--;
                                }
                            }
                            else
                            {
                                var Num = Random.Next(CountMatches / 2, CountMatches - 1);

                                while (!Array[Num])
                                {
                                    Array[Num] = true;
                                    CountByes--;
                                }
                            }
                            break;
                    }
                    Aux++;
                }
                return Array;
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private int SetRoundType(int IdRound, int Count, bool isFirst)
        {
            try
            {
                if (isFirst)
                {
                    if (Count == 4)
                        return 2;
                    else if (Count <= 8)
                        return 3;
                    else if (Count <= 16)
                        return 4;
                    else if (Count > 31 || Count <= 128)
                        return 7;
                    else
                        return 0;
                }
                else
                {
                    if (IdRound == 7 && Count <= 32)
                        return 4;
                    else if (IdRound == 7 && Count <= 64)
                        return 6;
                    else if (IdRound == 7 && Count <= 128)
                        return 6;
                    else if (IdRound == 6 && Count > 64)
                        return 5;
                    else if (IdRound == 1)
                        return 0;
                    else if (IdRound == 6 && Count > 32 && Count <= 64)
                        return IdRound - 2;
                    else if (IdRound == 7 && Count > 16 && Count <= 32)
                        return IdRound - 3;
                    else
                        return IdRound - 1;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private int GetRoundsTotal(int CountPlayers, int IdRound = 0)
        {
            try
            {
                if (IdRound != 0)
                {
                    switch (IdRound)
                    {
                        case 1:
                            return 1;
                        case 2:
                            return 2;
                        case 3:
                            return 4;
                        case 4:
                            return 8;
                        case 5:
                            return 16;
                        case 6:
                            if (CountPlayers <= 64)
                                return 16;
                            else
                                return 32;
                        default:
                            if (CountPlayers <= 32)
                                return 16;
                            else if (CountPlayers <= 64)
                                return 32;
                            else
                                return 64;
                    }
                }
                else
                {
                    if (CountPlayers == 4)
                        return 2;
                    else if (CountPlayers <= 8)
                        return 3;
                    else if (CountPlayers <= 16)
                        return 4;
                    else if (CountPlayers <= 32)
                        return 5;
                    else if (CountPlayers <= 64)
                        return 6;
                    else
                        return 7;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private int MatchesCount(int IdRound)
        {
            try
            {
                switch (IdRound)
                {
                    case 2:
                        return 2;
                    case 3:
                        return 4;
                    case 4:
                        return 8;
                    case 5:
                        return 16;
                    case 6:
                        return 32;
                    case 7:
                        return 64;
                    default:
                        return 0;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private int ByeCountFirstRound(int Count)
        {
            try
            {
                var Max = 64;

                while (Max > Count)
                {
                    Max /= 2;
                }
                return Max * 2 - Count;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private void SetNextRound(int MatchesFirstRound, ref List<Match> Maches)
        {
            try
            {
                var LastRoundLastMachNum = 0;
                var LastMachThisRoundNum = MatchesFirstRound;
                var AuxChangeRound = MatchesFirstRound;
                for (int i = 1; i < Maches.Count; i++)
                {
                    if (i > LastMachThisRoundNum)
                    {
                        LastRoundLastMachNum = LastMachThisRoundNum;
                        LastMachThisRoundNum = LastRoundLastMachNum + AuxChangeRound / 2;
                        AuxChangeRound /= 2;
                    }

                    var NextMachNum = (i - LastRoundLastMachNum) / 2 + LastMachThisRoundNum;
                    if (i % 2 != 0)
                        NextMachNum++;
                    Maches[i - 1].NextMatchTemp = NextMachNum;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private bool IsByeRound(int CountPlayers)
        {
            try
            {
                while (CountPlayers > 1 && CountPlayers % 2 == 0)
                    CountPlayers /= 2;

                if (CountPlayers == 1)
                    return false;
                else
                    return true;

            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion

        #region Show Draw
        [AllowAnonymous]
        [HttpGet]
        public ActionResult ShowDraw(int IdTournament, int IdCategory)
        {
            try
            {
                var Draw = DrawDAO.FindByTournamentAndCategory(IdTournament, IdCategory);
                var Rounds = Draw.Matches.Select(x => x.Round).Distinct().ToList();
                var RoundNames = Rounds.Select(x => x.NameRound).ToList();
                var CountRounds = Rounds.Max(x => x.IdRound);
                var Tournament = TournamentDAO.FindByIdWithPlayers(IdTournament);
                var CountPlayers = Tournament.EnrolledPlayers.Where(x => x.IdCategory == IdCategory).ToList().Count;
                var Category = CategoryDAO.GetDescript(Draw.IdCategory);

                var Bracket = new Bracket
                {
                    Rounds = new List<Rounds>(),
                    Names = RoundNames,
                    TournamentCategory = Category,
                    TournametName = Tournament.NameTournament,
                    TournamentDate = Tournament.StartDate.ToShortDateString() + " até " + Tournament.EndDate.ToShortDateString()
                };

                var IsFirstRound = true;
                var a = 0;

                foreach (var item in Rounds)
                {
                    var Round = new Rounds
                    {
                        Players = new List<RMatch>()
                    };
                    var LastRound = new Round();
                    var LastMatches = new List<Match>();
                    foreach (var match in item.Matches.OrderBy(x => x.Sequence))
                    {
                        var HasBye = match.MatchStats.Count == 1 ? true : false;
                        if (!IsFirstRound)
                        {
                            LastRound = Rounds.Where(x => x.IdRound == DrawGetLastRound(item.IdRound, CountPlayers)).FirstOrDefault();
                            LastMatches = LastRound.Matches.Where(x => x.IdNextMatch == match.IdMatch).ToList();
                        }

                        Round.Players.Add(GenerateMatch(match, IsFirstRound, HasBye, LastMatches));

                    }


                    Bracket.Rounds.Add(Round);
                    a++;
                    IsFirstRound = false;
                }

                Bracket.Rounds.Add(GetChampionRound(ref Rounds));

                var FinalBracket = JsonConvert.SerializeObject(Bracket);
                return View("Draw", Bracket);
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao obter informações das partidas - " + e.Message;
                return RedirectToAction("Index");
            }
        }

        private int DrawGetLastRound(int IdRound, int Count)
        {
            try
            {
                if (IdRound == 1)
                    return 2;
                else if (IdRound == 2)
                    return 3;
                else if (IdRound == 3)
                    return 4;
                else if (IdRound == 5)
                    return 6;
                else if (IdRound == 6)
                    return 7;
                else if (IdRound == 4)
                {
                    if (Count > 16 && Count <= 32)
                        return 7;
                    else if (Count > 32 && Count <= 64)
                        return 6;
                    else
                        return 5;
                }
                else return 0;
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private RMatch GenerateMatch(Match Match, bool IsFirstRound, bool HasBye, List<Match> LastMatches)
        {
            try
            {
                var Result = "";
                var RMatch = new RMatch();
                var FirstMatch = LastMatches.Where(x => x.Sequence % 2 != 0).FirstOrDefault();
                var SecondMatch = LastMatches.Where(x => x.Sequence % 2 == 0).FirstOrDefault();

                if (Match.MatchStats.Count > 0)
                {
                    var Player1 = new Stats();
                    var Player2 = new Stats();

                    if (!IsFirstRound && Match.MatchStats.Count > 1)
                    {
                        var FirstWinner = FirstMatch.MatchStats.Where(x => x.Result.Equals("W")).FirstOrDefault();
                        var SecondWinner = SecondMatch.MatchStats.Where(x => x.Result.Equals("W")).FirstOrDefault();

                        Result = FillResults(FirstMatch.MatchStats.ToList(), FirstWinner.IdPerson);

                        RMatch.player1 = FillPlayer1(FirstWinner, false, Result);

                        Result = FillResults(SecondMatch.MatchStats.ToList(), SecondWinner.IdPerson);

                        RMatch.player2 = FillPlayer2(SecondWinner, false, Result);
                    }

                    else if (!IsFirstRound && Match.MatchStats.Count == 1)
                    {
                        var IsFirstMatch = false;
                        var Winner = new Stats();
                        if (FirstMatch.MatchStats.Where(x => x.Result != null && x.Result.Equals("W")).FirstOrDefault() != null)
                        {
                            Winner = FirstMatch.MatchStats.Where(x => x.Result.Equals("W")).FirstOrDefault();
                            IsFirstMatch = true;
                        }
                        else
                            Winner = SecondMatch.MatchStats.Where(x => x.Result != null && x.Result.Equals("W")).FirstOrDefault();
                        var Pos = Winner.Match.Sequence % 2 == 0 ? 0 : 1;

                        if (Pos == 1)
                        {
                            Player1 = Match.MatchStats.Where(x => x.IdPerson == Winner.IdPerson).FirstOrDefault();

                            Player2 = null;
                        }
                        else
                        {
                            Player1 = null;
                            Player2 = Match.MatchStats.Where(x => x.IdPerson == Winner.IdPerson).FirstOrDefault();
                        }

                        if (IsFirstMatch)
                            Result = FillResults(FirstMatch.MatchStats.ToList(), Winner.IdPerson);
                        else
                            Result = FillResults(SecondMatch.MatchStats.ToList(), Winner.IdPerson);

                        RMatch.player1 = FillPlayer1(Player1, false, Player1 != null ? Result : "");


                        RMatch.player2 = FillPlayer2(Player2, false, Player2 != null ? Result : "");
                    }
                    else if (IsFirstRound)
                    {
                        Player1 = Match.MatchStats.ElementAt(0);

                        RMatch.player1 = FillPlayer1(Player1, false);

                        if (HasBye)
                            RMatch.player2 = FillPlayer2(null, true);
                        else
                        {
                            Player2 = Match.MatchStats.ElementAt(1);
                            RMatch.player2 = FillPlayer2(Player2, false);
                        }
                    }
                    else
                    {
                        RMatch.player1 = FillPlayer1(null, false);
                        RMatch.player2 = FillPlayer2(null, false);
                    }
                }
                else
                {
                    RMatch.player1 = FillPlayer1(null, false);
                    RMatch.player2 = FillPlayer2(null, false);
                }

                if (Match.DateMatch.HasValue)
                    RMatch.date = Match.DateMatch.Value.ToString("dd/MM/yyyy HH:mm");

                return RMatch;
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private string FillResults(List<Stats> Results, int IdPerson)
        {
            try
            {
                var P1 = Results.Where(x => x.IdPerson == IdPerson).FirstOrDefault();
                var P2 = Results.Where(x => x.IdPerson != IdPerson).FirstOrDefault();
                var Result = "";
                if (P1.FirstSetGames != 0)
                {
                    Result = P1.FirstSetGames.ToString() + "/" + P2.FirstSetGames.ToString() + " " + P1.SecondSetGames.ToString() + "/" + P2.SecondSetGames;
                    if (P1.ThirdSetGames != 0)
                        Result += " " + P1.ThirdSetGames.ToString() + "/" + P2.ThirdSetGames.ToString();
                }
                return Result;
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private Player1 FillPlayer1(Stats Player1, bool IsBye, string Result = "")
        {
            try
            {
                var P1 = new Player1();

                if (Player1 != null)
                {
                    P1.ID = Player1.IdPerson;
                    P1.name = Player1.Person.FullName;
                    if (!string.IsNullOrEmpty(Result))
                        P1.name += " " + Result;
                    P1.winner = !string.IsNullOrEmpty(Player1.Result) && Player1.Result.Equals("W") ? true : false;

                }
                else if (IsBye)
                {
                    P1.ID = -1;
                    P1.name = "BYE";
                    P1.winner = false;
                }
                else
                {
                    P1.ID = 0;
                    P1.name = "";
                    P1.winner = false;
                }

                return P1;
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private Player2 FillPlayer2(Stats Player2, bool IsBye, string Result = "")
        {
            try
            {
                var P2 = new Player2();

                if (Player2 != null)
                {
                    P2.ID = Player2.IdPerson;
                    P2.name = Player2.Person.FullName;
                    if (!string.IsNullOrEmpty(Result))
                        P2.name += " " + Result;

                    P2.winner = !string.IsNullOrEmpty(Player2.Result) && Player2.Result.Equals("W") ? true : false;

                }
                else if (IsBye)
                {
                    P2.ID = -2;
                    P2.name = "BYE";
                    P2.winner = false;
                }
                else
                {
                    P2.ID = 0;
                    P2.name = "";
                    P2.winner = false;
                }

                return P2;
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        private Rounds GetChampionRound(ref List<Round> AllRounds)
        {
            try
            {
                var Rounds = AllRounds;
                var FinalMatch = Rounds.Where(x => x.IdRound == 1).FirstOrDefault().Matches.FirstOrDefault();
                if (FinalMatch.MatchStats.Any(x => x.Result.Equals("W")))
                {
                    var oStats = FinalMatch.MatchStats.Where(x => x.Result.Equals("W")).FirstOrDefault();
                    var oRound = new Rounds();
                    var oRMatch = new RMatch
                    {
                        player1 = new Player1
                        {
                            ID = oStats.IdPerson,
                            name = oStats.Person.FullName,
                            winner = true
                        }
                    };
                    oRound.Players = new List<RMatch>
                    {
                        oRMatch
                    };
                    return oRound;
                }
                else
                {
                    var oRound = new Rounds();
                    var oRMatch = new RMatch
                    {
                        player1 = new Player1
                        {
                            ID = 0,
                            name = "",
                            winner = true
                        }
                    };
                    oRound.Players = new List<RMatch>
                    {
                        oRMatch
                    };
                    return oRound;
                }
            }
            catch (Exception e)
            {
                throw new Exception(ExceptionTreatment.GetExceptionDetails(e));
            }
        }

        #endregion
    }
}