﻿using Facebook;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using TpmDAO.Entity;
using TpmDAO.Persistence;

namespace TpmMVC.Controllers
{
    [AllowAnonymous]
    public class AccountController : BaseController
    {
        #region Properties

        private string ApiId { get; set; } = "311981692605799";

        private string AppSecret { get; set; } = "06799871aee2ed5d61adaf1c67960e88";

        private bool PremiumRequest { get; set; }

        private Uri RedirectUri
        {
            get
            {
                var uriBuilder = new UriBuilder(Request.Url);
                uriBuilder.Query = null;
                uriBuilder.Fragment = null;
                uriBuilder.Path = Url.Action("FacebookCallback");
                return uriBuilder.Uri;
            }
        }

        #endregion

        #region Facebook API

        public ActionResult Facebook(bool Premium = false)
        {
            PremiumRequest = Premium;
            Session["isPremium"] = PremiumRequest;
            var fb = new FacebookClient();
            var loginUrl = fb.GetLoginUrl(new
            {
                client_id = ApiId,
                client_secret = AppSecret,
                redirect_uri = RedirectUri.AbsoluteUri,
                response_type = "code",
                scope = "public_profile,email,user_birthday"
            });

            return Redirect(loginUrl.AbsoluteUri);
        }

        public ActionResult FacebookCallback(string code)
        {
            try
            {
                if (!string.IsNullOrEmpty(code))
                {

                    var fb = new FacebookClient();
                    dynamic result = fb.Post("oauth/access_token", new
                    {
                        client_id = ApiId,
                        client_secret = AppSecret,
                        redirect_uri = RedirectUri.AbsoluteUri,
                        code = code
                    });

                    fb.AccessToken = result.access_token;

                    // Get the user's information, like email, first name, middle name etc
                    dynamic me = fb.Get("me?fields=first_name,middle_name,last_name,id,email,locale,gender,birthday, picture.type(large)");
                    string email = me.email;
                    var User = UserDAO.FindByEmail(email);
                    if (User != null)
                    {
                        var IsPremium = (bool)Session["isPremium"];
                        Session.RemoveAll();
                        if (IsPremium)
                            return CheckPremiumRequest(User);
                        else
                            return CheckRole(User);

                    }
                    else
                    {
                        var FaceUser = FillFacebookData(me);
                        TempData["User"] = FaceUser;
                        return RedirectToAction("FacebookRegister", "UserLogin");
                    }
                }else
                    return RedirectToAction("Index", "UserLogin", new { msg = "Você precisa conceder acesso ao Facebook!" });

            }
            catch (Exception e)
            {
                return RedirectToAction("Index", "UserLogin", new { msg = "Erro ao acessar pelo Facebook! - " + e.Message});
            }
        }

        #endregion

        #region Private Methods 

        private ActionResult CheckPremiumRequest(User User)
        {
            try
            {
                var UserRoles = UserRoleDAO.GetRequestByUser(User.IdUser);
                if (User.UserRoles.Where(x => x.IdRole == 2 && !x.IsNew && x.Status).FirstOrDefault() != null)
                    return RedirectToAction("Index", "UserLogin", new { msg = "Você já possui uma conta premium!" });


                if (UserRoles != null)
                    return RedirectToAction("Index", "UserLogin", new { msg = "Você já solicitou uma conta premium, por favor aguarde a aprovação!" });
                var UserRole = new UserRole
                {
                    IdRole = 2,
                    IdUser = User.IdUser,
                    IsNew = true,
                    Status = false,
                };
                UserRoleDAO.Insert(UserRole);
                //SmtpServer.SendEmailWelcome(User.Email, "Solicitação de conta Premium");
                return RedirectToAction("Index", "UserLogin", new { msg = "Solicitação realizada com sucesso!", type = 1 });
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        private FacebookUser FillFacebookData(dynamic User)
        {
            try
            {
                var FaceUser = new FacebookUser
                {
                    FirstName = User.first_name,
                    MidleName = User.middle_name,
                    LastName = User.last_name,
                    Id = User.id,
                    Gender = User.gender,
                    Birthday = User.birthday,
                    Email = User.email,
                    PictureProfile = User.picture.data.url
                };

                return FaceUser;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion
    }
}