﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using TpmMVC.Models;
using TpmDAO.Persistence;
using TpmDAO.Entity;
using System.Text.RegularExpressions;
using TpmMVC.Util;
using System.Threading.Tasks;
using System.IO;
using Newtonsoft.Json;
using System.Globalization;
using Utils;

namespace TpmMVC.Controllers
{
    [AllowAnonymous]
    public class UserLoginController : BaseController
    {

        #region Views

        public ActionResult Index(string msg = "", int type = 0)
        {
            if (!string.IsNullOrEmpty(msg))
            {
                if (type == 0)
                    TempData["Error"] = msg;
                else
                    TempData["Success"] = msg;
            }
            if (User.Identity.IsAuthenticated)
            {
                var User = SessionManager.GetUserData();
                return CheckRole(User);
            }
            return View();
        }

        public ActionResult Register()
        {
            ViewBag.ShowAddress = false;
            FillDropDownsUser();
            return PartialView("_Register");
        }

        public ActionResult ForgotPassword()
        {
            return PartialView();
        }

        public ActionResult PremiumRequest()
        {
            return PartialView("_PremiumRequest");
        }

        #endregion

        #region Login

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult SignUp(LoginViewModel model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var User = UserDAO.Autenticate(model.Login, Encryption.GetMD5Hash(model.Password));
                    if (User != null)
                    {
                        User.Token = null;
                        return CheckRole(User);
                    }
                    else
                    {
                        return RedirectToAction("Index", new { msg = "Acesso Negado!" });
                    }
                }
                else
                    return View("Index", model);
            }
            catch (Exception e)
            {
                return RedirectToAction("Index", new { msg = "Erro ao efetuar login  - " + e.Message });
            }
        }

        [HttpGet]
        public ActionResult LogOut()
        {
            SessionManager.SignOut();
            return RedirectToAction("Index");
        }

        #endregion

        #region Forgot Password

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult ForgotPassword(ForgotPasswordModel model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var User = UserDAO.FindByEmail(model.Email);

                    if (User != null && string.IsNullOrEmpty(User.FacebookId))
                    {
                        User.Token = Token.GenerateSafeToken();
                        UserDAO.Update(User);

                        SmtpServer.SendEmailResetPassword(User.Email, PersonDAO.FindByKey(User.IdPerson).FullName, User.Token);
                        return GetSuccess("Foi enviado ao seu email, um link para redefinição de senha!");
                    }
                    else if (User != null && !string.IsNullOrEmpty(User.FacebookId))
                        return GetWarning("Cadastro realizado com Facebook, Recupere sua senha no site do Facebook!");
                    else
                        return GetError("Usuário não encontrado!");
                }
                else
                    return PartialView("ForgotPassword", model);
            }
            catch (Exception e)
            {
                return GetError("Erro ao enviar link de redefinição de senha! - " + e.Message);
            }
        }

        [HttpGet]
        public ActionResult ResetPassword(string Token)
        {
            try
            {
                var User = UserDAO.FindUserByToken(Token);
                if (User != null)
                {
                    var oToken = Utils.Token.GetTokenInfo(Token);
                    if (oToken != null)
                    {
                        if (oToken.Date < DateTime.Now.AddHours(-72))
                        {
                            TempData["Warning"] = "O Token expirou, solicite uma nova redefinição de senha!";
                            return RedirectToAction("Index");
                        }
                        else
                        {
                            var Model = new ResetPasswordViewModel
                            {
                                IdUser = User.IdUser.ToString(),
                                UserName = User.Person.FullName,
                                Code = Token
                            };
                            return View(Model);
                        }
                    }
                }
                TempData["Error"] = "Token de segurança inválido!";
                return RedirectToAction("Index");
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao validar token! - " + e.Message;
                return RedirectToAction("Index");
            }
        }

        [HttpPost]
        public ActionResult ResetPassword(ResetPasswordViewModel Model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var User = UserDAO.FindByKey(Convert.ToInt32(Model.IdUser));
                    User.Password = Encryption.GetMD5Hash(Model.Password);
                    UserDAO.Update(User);

                    TempData["Success"] = "Senha redefinida com sucesso!";
                    return RedirectToAction("Index");
                }
                else
                    return View(Model);
            }
            catch (Exception e)
            {
                TempData["Error"] = "Erro ao altertar senha! - " + e.Message;
                return RedirectToAction("Index");
            }
        }

        #endregion

        #region Premium Request

        [HttpPost]
        public ActionResult PremiumRequest(LoginViewModel model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var User = UserDAO.Autenticate(model.Login, Encryption.GetMD5Hash(model.Password));
                    if (User != null)
                    {
                        var UserRoles = UserRoleDAO.GetRequestByUser(User.IdUser);
                        if (UserRoles != null) {
                            if (UserRoles.IsNew)
                                return GetInfo("Você já solicitou uma conta premium, por favor aguarde a aprovação");
                            else if (!UserRoles.Status)
                            {
                                UserRoles.IsNew = true;
                                UserRoles.Status = true;
                                UserRoleDAO.Update(UserRoles);
                            }
                                }
                        else {
                            var UserRole = new UserRole
                            {
                                IdRole = 2,
                                IdUser = User.IdUser,
                                IsNew = true,
                                Status = false,
                            };
                            UserRoleDAO.Insert(UserRole);
                        }

                        SmtpServer.SendEmailRequest(User.Email, User.Person.FullName, 0);

                        return GetSuccess("Socilitação realizada com sucesso");
                    }
                    else
                        return GetError("Usuário não encontrado!");
                }
                else
                    return PartialView("_PremiumRequest");
            }
            catch (Exception e)
            {
                return GetError("Erro ao realizar solicitação - " + e.Message);
            }
        }

        #endregion

        #region Register

        public ActionResult FacebookRegister()
        {
            try
            {
                var UserData = (FacebookUser)TempData["User"];

                FillDropDownsUser();
                var Model = FillFacebookModel(UserData);
                return View("FacebookRegister", Model);
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        [ValidateAntiForgeryToken]
        [HttpPost]
        public ActionResult FacebookRegister(FacebookRegisterModel Model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var User = FillUserRegister(Model);
                    UserDAO.Insert(User);
                    SmtpServer.SendEmailWelcome(Model.Email, User.Person.FullName);
                    TempData["Success"] = "Cadastro realizado com sucesso! \nSolicite uma conta premium para ter acesso ao nosso sistema web!";
                    return RedirectToAction("Index");
                }
                else
                {
                    FillDropDownsUser(Convert.ToInt32(Model.State));
                    return View("FacebookRegister", Model);
                }
            }
            catch (Exception e)
            {
                if (e.HResult == -2146233088)
                {
                    TempData["Error"] = "Usuario Cadastrado!\nFalha ao enviar email de confirmação!";
                    return PartialView("FacebookRegister", Model);
                }

                TempData["Error"] = "Erro ao cadastrar usuário! - " + e.Message;
                FillDropDownsUser(Convert.ToInt32(Model.State));
                return View("FacebookRegister", Model);
            }
        }

        [HttpPost]
        public async Task<ActionResult> Register(UserRegisterModel model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var User = await FillUserRegister(model);
                    UserDAO.Insert(User);
                    SmtpServer.SendEmailWelcome(User.Email, User.Person.FullName);
                    return GetSuccess("Cadastro realizado com sucesso!");
                }
                else
                {
                    ViewBag.ShowAddress = true;
                    FillDropDownsUser(Convert.ToInt32(model.State));
                    return PartialView("_Register", model);
                }
            }
            catch (Exception e)
            {
                if (e.HResult == -2146233088)
                {
                    TempData["Error"] = "Usuario Cadastrado!\nFalha ao enviar email de confirmação!";
                    return PartialView("_Register", model);
                }
                ViewBag.ShowAddress = true;
                FillDropDownsUser();
                TempData["Error"] = "Erro ao cadastrar usuário - " + e.Message;
                return PartialView("_Register", model);
            }
        }

        #endregion

        #region Fill User Object

        private async Task<User> FillUserRegister(UserRegisterModel model)
        {
            var FileName = "";
            try
            {
                #region User

                var User = new User
                {
                    Email = model.Email,
                    Password = Encryption.GetMD5Hash(model.Password)
                };

                #endregion

                #region Person

                User.Person = new Person
                {
                    Address = new Address(),
                    BirthDate = model.BirthDate,
                    Class = CategoryClassDAO.FindByKey(Convert.ToInt32(model.IdClass)).Class,
                    FirstName = model.Name,
                    LastName = model.LastName,
                    Phone = RemoveCharsNonNumeric(model.Tel),
                    Gender = model.Gender,
                    IdHand = Convert.ToInt32(model.IdHand),
                    TotalStats = new TotalStats()
                };

                #endregion

                #region Address

                User.Person.Address.IdCity = Convert.ToInt32(model.City);
                User.Person.Address.Street = model.Street;
                User.Person.Address.Detail = model.Detail;
                User.Person.Address.Neighborhood = model.Neighborhood;
                User.Person.Address.Number = model.Number;
                User.Person.Address.ZipCode = RemoveCharsNonNumeric(model.Cep);

                #endregion

                #region Role
                var UserRole = new UserRole
                {
                    IdRole = 3,
                    IdUser = User.IdUser,
                    IsNew = false,
                    Status = true,
                    RequisitionDate = DateTime.Now,

                };
                User.UserRoles = new List<UserRole>
                {
                    UserRole
                };
                #endregion

                FileName = model.Photo != null ? Guid.NewGuid().ToString() + Path.GetExtension(model.Photo.FileName) : "";
                User.ProfilePicture = model.Photo != null ? await Images.UploadPicture(model.Photo, FileName, "profile_pic") : null;
                FileName = User.ProfilePicture;
                return User;
            }
            catch (Exception e)
            {
                if (!FileName.Equals(""))
                    await Images.DeletePicture(FileName, "profile_pic");
                throw new Exception(e.Message);
            }
        }

        private User FillUserRegister(FacebookRegisterModel model)
        {
            try
            {
                #region User

                var User = new User
                {
                    Email = model.Email,
                    FacebookId = model.FacebookId
                };

                User.ProfilePicture = model.ProfilePicture;
                #endregion

                #region Person

                User.Person = new Person
                {
                    Address = new Address(),
                    BirthDate = DateTime.Parse(model.BirthDateFormat, CultureInfo.InvariantCulture),
                    Class = CategoryClassDAO.FindByKey(Convert.ToInt32(model.IdClass)).Class,
                    FirstName = model.Name,
                    LastName = model.LastName,
                    Phone = RemoveCharsNonNumeric(model.Tel),
                    Gender = model.Gender,
                    IdHand = Convert.ToInt32(model.IdHand),
                    TotalStats = new TotalStats()
                };

                #endregion

                #region Address

                User.Person.Address.IdCity = Convert.ToInt32(model.City);
                User.Person.Address.Street = model.Street;
                User.Person.Address.Detail = model.Detail;
                User.Person.Address.Neighborhood = model.Neighborhood;
                User.Person.Address.Number = model.Number;
                User.Person.Address.ZipCode = RemoveCharsNonNumeric(model.Cep);

                #endregion

                #region Role
                var UserRole = new UserRole
                {
                    IdRole = 3,
                    IdUser = User.IdUser,
                    IsNew = false,
                    Status = true,
                    RequisitionDate = DateTime.Now,

                };
                User.UserRoles = new List<UserRole>
                {
                    UserRole
                };
                #endregion

                return User;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

        #region Fill FacebookData

        private FacebookRegisterModel FillFacebookModel(FacebookUser User)
        {
            try
            {
                var Model = new FacebookRegisterModel
                {
                    Name = User.FirstName + " " + User.MidleName,
                    LastName = User.LastName,
                    BirthDateFormat = string.IsNullOrEmpty(User.Birthday) ? "" : DateTime.Parse(User.Birthday, CultureInfo.InvariantCulture).ToString("yyyy-MM-dd"),
                    Email = User.Email,
                    Gender = User.Gender.Equals("male") ? "M" : User.Gender.Equals("female") ? "F" : "M",
                    ProfilePicture = User.PictureProfile,
                    FacebookId = User.Id
                };
                return Model;
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        #endregion

    }
}