﻿using System;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Optimization;
using System.Web.Routing;
using System.Web.Security;
using Newtonsoft.Json;

namespace TpmMVC
{
    public class MvcApplication : HttpApplication
    {
        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();
            FilterConfig.RegisterGlobalFilters(GlobalFilters.Filters);
            RouteConfig.RegisterRoutes(RouteTable.Routes);
            BundleConfig.RegisterBundles(BundleTable.Bundles);
        }

        public override void Init()
        {
            PostAuthenticateRequest += new EventHandler(MvcApplication_PostAuthenticateRequest);
            base.Init();
        }

        void MvcApplication_PostAuthenticateRequest(object sender, EventArgs e)
        {
            var authCookie = HttpContext.Current.Request.Cookies[FormsAuthentication.FormsCookieName];

            if (authCookie != null)
            {
                string encTicket = authCookie.Value;
                if (!String.IsNullOrEmpty(encTicket))
                {
                    var ticket = FormsAuthentication.Decrypt(encTicket);
                    var id = new Controllers.UserIdentity(ticket);
                    var userRoles = Roles.GetRolesForUser(id.Name);
                    var prin = new GenericPrincipal(id, userRoles);
                    HttpContext.Current.User = prin;
                }
            }
        }

        protected void Application_Error()
        {
            HttpContext httpContext = HttpContext.Current;
            if (httpContext != null)
            {
                var StatusCode = 0;
                IController controller = new Areas.RestrictedArea.Controllers.HomeController();
                var exception = Server.GetLastError();
                if (exception is HttpException httpException)
                {
                    StatusCode = httpException.GetHttpCode();
                }
                var Inner = exception.InnerException != null ? exception.InnerException.Message : "";
                RequestContext requestContext = ((MvcHandler)httpContext.CurrentHandler).RequestContext;
                httpContext.Response.Clear();
                ControllerContext controllerContext = new ControllerContext(requestContext, (ControllerBase)controller);

                #region AJAX Request

                if (requestContext.HttpContext.Request.IsAjaxRequest())
                {
                    JsonResult jsonResult = new JsonResult
                    {
                        Data = new { Type = 2, Message = "Error " + StatusCode + " - " + exception.Message + " " + Inner },
                        JsonRequestBehavior = JsonRequestBehavior.AllowGet
                    };
                    jsonResult.ExecuteResult(controllerContext);
                    httpContext.Response.End();
                }

                #endregion

                #region Action Request

                else
                {
                    ActionResult Result =
                        new RedirectToRouteResult(new RouteValueDictionary(new { area = "", action = "Error", controller = "Base", msg = "Error " + StatusCode + " - " + exception.Message + " " + Inner }));
                    Result.ExecuteResult(controllerContext);
                    httpContext.Response.End();
                }

                #endregion
            }
        }

    }
}
