﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Security;
using TpmDAO.Entity;
using Utils;

namespace TpmMVC.Util
{
    public class SessionManager
    {

        public static void SetCookieToUser(User user)
        {
            try
            {
                user.UserRoles = null;
                user.Password = null;
                var oTimeOut = DateTime.Now.AddMinutes(120);
                string userData = Encryption.Encrypt(JsonConvert.SerializeObject(user, new JsonSerializerSettings
                { ReferenceLoopHandling = ReferenceLoopHandling.Ignore, NullValueHandling = NullValueHandling.Ignore }));
                var authTicket = new FormsAuthenticationTicket(1, user.Email, DateTime.Now, oTimeOut, false, userData);

                var encTicket = FormsAuthentication.Encrypt(authTicket);
                HttpContext.Current.Response.Cookies.Add(new HttpCookie(FormsAuthentication.FormsCookieName, encTicket));
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
        }

        public static void SignOut()
        {
            FormsAuthentication.SignOut();
        }

        public static User GetUserData()
        {
            try
            {
                HttpCookie authCookie = HttpContext.Current.Request.Cookies[FormsAuthentication.FormsCookieName];
                if (authCookie != null)
                {
                    FormsAuthenticationTicket ticket = FormsAuthentication.Decrypt(authCookie.Value);
                    return JsonConvert.DeserializeObject<User>(Encryption.Decrypt(ticket.UserData));
                }
                return null;
            }
            catch (Exception e)
            {
                throw new Exception("Erro ao obter dados do usuário - " + e.Message);
            }

        }
    }
}